/*

This is the server program to control the power switch.  It will be 
activated by any TCP/IP client program with the appropriate commands,
"ON" and "OFF".  Several stages are designed for this process.

Stage 1:  Just get it working.  Have a client send the ON and OFF commands
and activate or deactivate the switch accordingly. : DONE

Stage 2:  Put in a timer to automatically turn off the switch after a 
certain specified time frame, probably 5 minutes, to prevent excessive
power consumption. : DONE

Stage 3:  Built a better circut.  Each parallel port can control 8 switches.
Actually, each port CAN support 12, but the more conventional method is to
use the 8 data ports.  Each switch requires a resistor, a transistor, a
diode, and a relay.  The part total for each switch is approximately $15.

Stage 4:  Build a circut which can apply variable voltage, controlled by
the parallel port.  This will be necessary for controlling a remote control
mechanism for an RC car, which is the next big project.


NOTE:  You have to be root to run this server, or otherwise have permissions
to directly access the parallel port.


This server has now been modified to operate a stepper motor.  It relies on
knowing the current state of the parallel port, and therefore is controlled
by the same process as the light is.  

Motor control is a bit more complicated.  The stepper motor in use was
obtained from a discarded floppy drive.  Each "step" turns the shaft 1.8
degrees.  The motor turns by applying voltage to different contacts on the
motor in a specific order, and the reverse order will turn the shaft in the
other direction.  

An additional constraint is that the motor must not be allowed to move beyond
upper and lower bounds.  Although the motor can handle infinite turns in any
direction, the camera mounted to it is connected by a cable and the rotation
bounds must be limited to prevent the cable from getting wrapped around the
shaft.  A 90-180 degree total field of vision should be adequate for any 
camera located on or near a wall.

If you're banging your head over how to successfully wire the stepper motor
to work properly with my code, you may wish to reorganize the code.  I have
the numbers out of sequence because I have the stepper motor contacts
wired out of sequence and I adjusted the program to match it.  When I 
move off of the breadboard and onto a more permanant PC board, I will 
rewire everything correctly and fix the program then.

To step the motor, it is important to know which contact was tapped previously,
unless a minimum of 7.2 degree turns is desired (in which case all 4 contacts
can be tapped in one command).

The motor, since it can be moved manually, should be calibrated each time
the server starts or any time the motor is moved inadvertantly.  A simple
calibration will be for the user to manually point the camera at the center
of the bounded region, then send a command to set the currentstep value
accordingly.  The calibration command can come from the client or the
server as an argument when it starts up.  Right now the program simply
assumes that the camera is positioned in the centre of the field of vision
and sets the currentstep value at the midpoint.


This program has now been modified to operate a remote control car.  I 
accomplished this by wiring relays to the remote control unit of the car.
These relays are controlled the same way the lamp relay is.  This project
consumed an additional 4 transistors.

Please send any relavent comments, suggestions, complaints, but no spam to
restil@alignment.net.  This code works for me.  I claim nothing more.  
Especially if you are somewhat new to interfacing electronics with the 
computer, try to use a computer that you can afford to damage.  I use a 486/66
that I got for $30.  Its adaquate for this project, and with networking, 
I don't have to put my primary servers in any danger.

You are free to modify, redistribute, sell, whatever this code.  If you make
useful changes, feel free to update me with those changes, I'd appreciate
it.


Now modded to include commands to operate the X10 stuff.  This program will
keep track of the time limits, as well has have different time limits for
those devices, AND the lamp.  Now we get 3 minutes for teh internet peeps and
longer durations for local selections.


New remake of this program:

Use configuration files for all devices.

New multithreaded environment for handling longer lasting actions.

Return status data from all commands, saving a second call for status data.

Redirect option to send a signal to another server for other housecodes.


*/

/*  Strike list
These are a list of the strike values and what they instruct the browser to do.

0 - do nothing.
1 - Load a popup window with the waitqueue data
2 - Silent state.... whatever that was supposed to be. :)
3 - Load a popup window with the waiting message
4 - Load a popup window with the waiting URL




*/



#include <sys/types.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <sys/wait.h>
#include <sys/ioctl.h>
#include <ctype.h>
#include <limits.h>
#include <string.h>
#include <fcntl.h>
#include <netinet/in.h>
#include <signal.h>
#include <netdb.h>
#include <stdio.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <asm/io.h>
#include "x10.h"
#include "dmiheaders.h"

/*
all this data will be definied in config files, although defaults remain
*/
#define base 0x378
#define base2 0x278
#define VERYSLOW 100000
#define SLOW 200000
#define MEDIUM 250000
#define FAST 350000

#define DATA "You got it from me"


child_handler(int arg)
{
int     status;
wait(&status);
}        

pipecleaner(int arg)
{
}

unsigned char devicecode_table[] = {
        DEVICECODE_1,
        DEVICECODE_2,
        DEVICECODE_3,
        DEVICECODE_4,
        DEVICECODE_5,
        DEVICECODE_6,
        DEVICECODE_7,
        DEVICECODE_8,
        DEVICECODE_9,
        DEVICECODE_10,
        DEVICECODE_11,
        DEVICECODE_12,
        DEVICECODE_13,
        DEVICECODE_14,
        DEVICECODE_15,
        DEVICECODE_16
};



unsigned char housecode_table[] = {
        HOUSECODE_A,
        HOUSECODE_B,
        HOUSECODE_C,
        HOUSECODE_D,
        HOUSECODE_E,
        HOUSECODE_F,
        HOUSECODE_G,
        HOUSECODE_H,
        HOUSECODE_I,
        HOUSECODE_J,
        HOUSECODE_K,
        HOUSECODE_L,
        HOUSECODE_M,
        HOUSECODE_N,
        HOUSECODE_O,
        HOUSECODE_P
};

struct	waitqueue
	{
	char	host[80];
	int	priority;	//0 is lowest
	long	starttime;	// time entered the queue
	long	lastupdate;	// time of the last heartbeat check
	struct	waitqueue	*next;
	};

struct	messagequeue
	{
	char	message[2000];
	char	host[80];
	int	striketoggle; 	// The popup request has been sent
	int	readtoggle;	// The message has been read.
	long	sendtime;	// Time the message was sent.
	struct	messagequeue *next;
	};

struct	rccareventqueue
	{
	char	host[80];
	int	type;	//0 - power, 1 - turn
	int	direction; // f/b, or 0,1,2 (left, right, center)
	int	speed;
	struct	rccareventqueue *next;
	};

struct	x10eventqueue
	{
	char	host[20];
	char	buf[256];
	struct	x10eventqueue *next;
	};


struct	switchdata
	{
	char	name[20];
	char	label[40];	// name displayed
	int	switchtype;	// 0 - normal, 1 - push on/push off, 2 - toggle
	int	port;
	int	status;
	int	available;
	int	switchbase;
	int	control;
	int	counter;
	int	watts;		// watts used by this appliance.
	int	stateposition;	// position in the state variable
// timechecks
	int	timeout;
	int	startstatus;	// State the device is at midnight
	long	times[5];	// toggle availability at these times
	int	maxsecs;	// maximum minutes per day
	int	timeused;	// seconds used so far today.
	int	lock;		// is this locked?
	char	lockhost[20];	// Who holds the lock?
	long	lockexp;	// When does the lock expire?
	char	relayhost[30];	// relay host
	int	relayport;	// relay port
	char	relayname[20];	// name to use.
	int	relaythread;
	int	icon;		// 0 - bulbs, 1 - sprinkler, 2 - MS, 3- webcam
	struct	waitqueue *wq;
	char	camname[5][80];
	int 	bored;
	};

struct	sdlink
	{
	struct	switchdata	sd;
	struct	sdlink 		*next;
	};

struct motordata
	{
	char	name[20];
	char	label[40];
	int	sequence[4];
	int	zoominpin;
	int	zoomoutpin;
	int	zoomposition;
	int	zoomsteptime; // time in useconds for each step.
	int	numsequence;
	int	motorbase;
	int	currentstep;
	int	lastcontact;
	int	available;
	int	control;
	int	lock;		// is this locked?
	char	lockhost[20];	// Who holds the lock?
	long	lockexp;	// When does the lock expire?
	char	relayhost[30];	// relay host
	int	relayport;	// relay port
	char	relayname[40];
	int	relaythread;
	struct	waitqueue *wq;
	char	camname[5][80];
	};

struct	mdlink
	{
	struct	motordata	md;
	struct	mdlink 		*next;
	};


struct	camcount_type
	{
	int	cam;
	char	host[40];
	long	lastupdate;
	};

struct	cclink
	{
	struct	camcount_type	cc;
	struct	cclink	*next;
	};

struct rccardata
	{
	char	name[20];
	char	label[40];
	int	type;	// 0 - normal . 1 - minimal
	int	commonport;
	int	forwardport;
	int	backwardport;
	int	leftport;
	int	rightport;
	int	timeout;
	int	carbase;
	int	lock;		// is this locked?
	char	lockhost[20];	// Who holds the lock?
	long	lockexp;	// When does the lock expire?
	long	heartbeat;
	int	nudgef; // time in microseconds to delay
	int	nudgeb;
	int	pushf;
	int	pushb;
	int	wstatus;
	long	wcounter;
	int	available;
	int	control;
	char	relayhost[30];	// relay host
	int	relayport;	// relay port
	char	relayname[20];
	int	relaythread;
	struct	waitqueue *wq;
	struct	rccareventqueue *eq;
	char	camname[5][80];
	};

struct	rclink
	{
	struct	rccardata	rc;
	struct	rclink 		*next;
	};

struct	X10data
	{
	char	label[40];
	int	state;
	int	available;
	int	limit;
	int	counter;	
	int	timeused;
	int	remotetimeout;
	int	localtimeout;
	int	watts;
	int	stateposition;	// position in the state variable
	int	startstatus;	// State the device is at midnight
	int	type;		// 1 - br, 2 - ppower
	long	times[5];	// toggle availability at these times
	int	maxsecs;	// maximum minutes per day
	int	lock;		// is this locked?
	char	lockhost[20];	// Who holds the lock?
	long	lockexp;	// When does the lock expire?
	int	control;	// does this server control this module?
	char	relayhost[30];	// relay host
	int	relayport;	// relay port
	char	relayname[20];
	int	relaythread;
	int	relayhouse;	// housecode to relay this module to.
	int	icon;		// 0 - bulbs, 1 - sprinkler, 2 - MS, 3- webcam
	int	bored;		// put this on bored.com?
	struct	waitqueue *wq;
	char	camname[5][80];
	};


struct	bandata
	{
	char	host[20];
	long	banexp;		// When does this ban expire?
	struct	bandata *next;
	};	

struct	banwatch
	{
	char	host[20];
	int	count;		// number of hits
	long	timestamp;	// last time something happened
	int	warned;		// have we warned this user yet?
	};

struct	slaveinfo
	{
	int	status;
	int	msgsock;	// socket descriptor
	char	host[40];
	int	pongwait;	// Are we waiting for a pong?
	long	lastping;	// Last time we sent a ping.
	struct	slaveinfo	*next;
	int	getdevstat;	// did I get device data from the server?
	int	postdevstat;	// did I post device data to the server?
	};



int	pipes[2];
struct	sdlink	*switches;
struct	mdlink	*motors;
struct	rclink	*rccars;
struct	cclink	*camcounts;
struct	sklink	*socketlink;
struct	X10data	X10mods[17][17];
struct	x10eventqueue *x10eq;
int	X10state[17];
int	X10limit[17];
int	X10counter[17];
int	steps[4];  // sequence for stepper motor data pins

int	listenport;
int	bancheck;	// do we record/check for bans?
int	togglemax;	// max number of toggles before a ban
int	banhours;	// number of hours to enact a ban for.
int	bandeduct;	// number of seconds between one point reductions.
float	kwhcost;	// power cost per kwh

long	lastbanupdate;

int	locks[5];	// Seconds that locks last for each tier;

unsigned char	state;
unsigned char	state2;

struct	banwatch	banwatchlist[200];
struct	banwatch	rccarwatchlist[200];
struct	bandata		*bans;

struct	messagequeue	*mqlist;

struct	slaveinfo	slavedata[100];
struct	slaveinfo	myslave;
int	numslaves;
int	slaved;

char	brport[256];
char	ppowerpath[256];

char	commandsbuf[30000];

char	clientname[10];



int	ifbanned(char *host)
{
struct	bandata	*temp;
temp = bans;
while(temp != NULL)
	{
	if(strcmp(temp->host,host) == 0)
		return(1);
	temp = temp->next;
	}
return(0);
}

int	bancheckfunc(struct banwatch *banwatchlist, char *host,int togglemax)
{
int	x,y,z;
long	timestamp;
int	tsmark;
char	s[1024];

if(ifbanned(host))
	return(1);
// check for an open space, otherwise get the oldest timestamp;
timestamp = time(NULL);
tsmark = 0;
for(z=0;z<200;z++)
	{
	if(strcmp(banwatchlist[z].host,host) == 0)
		{
		banwatchlist[z].count++;
		if(banwatchlist[z].count > togglemax)
			addban(host);
		if(banwatchlist[z].count > (togglemax/2) && !banwatchlist[z].warned)
			{
			// send the user a warning message.
			sprintf(s,"This is an automated message:  Please slow down.  Continued abuse of the equipment will result in an automatic ban.");
			banwatchlist[z].warned = 1;
			messageenqueue(s,host);
			}
		}
	}
for(z=0;z<200;z++)
	{
	if(banwatchlist[z].host[0] == 0 || banwatchlist[z].count == 0)
		{
		strcpy(banwatchlist[z].host,host);
		banwatchlist[z].count=1;
		banwatchlist[z].warned=0;
		banwatchlist[z].timestamp = time(NULL);	
		return(0);
		}
	if(banwatchlist[z].timestamp < banwatchlist[tsmark].timestamp)
		tsmark = z;
	}
strcpy(banwatchlist[tsmark].host,host);
banwatchlist[z].count=1;
banwatchlist[z].warned=1;
banwatchlist[z].timestamp = time(NULL);
return(0);
}

int	addban(char *host)
{
int	x,y,z;
long	timestamp;
int	tsmark;
struct	bandata	*holder;
struct	bandata	*temp;

holder = malloc(sizeof *holder);
strcpy(holder->host,host);
holder->banexp = time(NULL) + banhours * (60*60);
holder->next=NULL;

if(bans==NULL)
	{
	bans=holder;
	}
else
	{
	temp=bans;
	while(temp->next != NULL)
		temp = temp->next;
	temp->next = holder;
	}
}

deletebans(struct banwatch *banwatchlist)
{
struct	bandata	*holder;
struct	bandata	*temp;
int	x,y,z;
if(time(NULL) > lastbanupdate + bandeduct)
	{
	lastbanupdate = time(NULL);
	for(z=0;z<200;z++)
		{
		if(banwatchlist[z].count > 0)
			banwatchlist[z].count--;
		}
	}
while(bans != NULL)
	{
	holder = bans;
	if(holder->banexp < time(NULL))
		{
		bans = holder->next;
		free(holder);
		}
	else
		return(0);
	}
}



// This function will take the base value and return the pointer to the
// correct state variable
unsigned char *getstatevar(int	basevar)
{
if(basevar == base)
	return(&state);
if(basevar == base2)
	return(&state2);
}

int	rccardequeue(struct rccardata *rccar)
{
struct	rccareventqueue *temp;
int	x,y,z;

temp = rccar->eq;
if(temp == NULL)
	{
	return(0);
	}
printf("Dequeueing car command: %d %d %d\n",temp->type, temp->speed, temp->direction);
rccar->eq = rccar->eq->next;
rccar->heartbeat = time(NULL);
if(temp->type == 0)
	power(temp->speed,temp->direction, rccar, getstatevar(rccar->carbase));
else if(temp->type == 1)
	turn(temp->direction, rccar, getstatevar(rccar->carbase));
else
	powerturn(temp->direction >> 2, temp->direction & 3, temp->speed, rccar);
free(temp);
z=0;
temp=rccar->eq;
while(temp != NULL)
	{
	temp = temp->next;
	z++;
	}
printf("%d commands left in the queue.\n",z);
}

int rccarenqueue(int type, int speed, int direction, char *host, struct	rccardata *rccar)
{
struct	rccareventqueue *temp;
struct	rccareventqueue *holder;
char	s[2000];
/*
if(bancheck)
	if(bancheckfunc(banwatchlist, host,200))
		return(0);
	if(bancheckfunc(rccarwatchlist, host,20))
		return(0);
*/
// First, check to see if the car is locked.  If so, add to the queue.
if(rccar->lock && strcmp(rccar->lockhost,host) != 0)
	{
	printf("Car is currently locked by .%s.  %s is being added to the waitqueue.\n",rccar->lockhost,host);
	rccarwaitenqueue(host,rccar);
	return(1);
	}
if(rccar->control == 3)
	{
	printf("I GOT HERE!!!!!!!!\n");
	// we send an ON request to the slave.
	if(speed == 1 && direction == 1 && type == 0)
		sprintf(s,"%s PUSHF 0 spaz\n",rccar->relayname);
	if(speed == 0 && direction == 2 && type == 2)
		sprintf(s,"%s DOWNRIGHT 0 spaz\n",rccar->relayname);
	if(speed == 0 && direction == 0 && type == 1)
		sprintf(s,"%s CENTER 0 spaz\n",rccar->relayname);
	printf("s = .%s.\n",s);
	if(rccar->relaythread >= 0)
		write(slavedata[rccar->relaythread].msgsock,s,strlen(s)+1); 	
	else
		write(myslave.msgsock,s,strlen(s)+1); 	
	return(1);
	}
holder = malloc(sizeof *holder);
strcpy(holder->host,host);
holder->type = type;
holder->speed = speed;
holder->direction = direction;
holder->next = NULL;
temp = rccar->eq;
if(temp == NULL)
	rccar->eq = holder;
else
	{
	if(strcmp(host,temp->host) == 0 && strcmp(host,"") != 0)
		{
		free(holder);
		return(0);
		}
	while(temp->next != NULL)
		{
		if(strcmp(host,temp->host) == 0 && strcmp(host,"") != 0)
			{
			free(holder);
			return(0);
			}
		temp = temp->next;
		}
	temp->next = holder;
	printf("Enqueuing. host: %s\n",host);
	}
return(0);
}


int	messageenqueue(char	*message, char *host)
{
struct	messagequeue	*temp;
struct	messagequeue	*holder;

printf("Enqueuing message. \n");
holder = malloc(sizeof *holder);
printf("Message: %s    host: %s\n",message,host);
strcpy(holder->message,message);
strcpy(holder->host,host);
holder->readtoggle = 0;
holder->striketoggle =0;
time(&holder->sendtime);
holder->next = NULL;

temp = mqlist;
if(temp == NULL)
	mqlist = holder;
else
	{
	while(temp->next != NULL)
		temp = temp->next;
	temp->next = holder;
	}
}

int	messagestrike(char *host)
{
int	x,y,z;
struct	messagequeue	*temp;

printf("Checking for message strikes.\n");
temp = mqlist;
x=0;
while(temp != NULL)
	{
	if(strcmp(host,temp->host) == 0 && temp->striketoggle == 0)
		{
		printf("Found strike for: %s  on queue #%d\n",temp->host,x);
		temp->striketoggle = 1;
		return(3);
		}	
	temp = temp->next;
	x++;
	}
return(0);
}

int	messagedequeue(char *buf, char *host)
{
int	x,y,z;
struct	messagequeue *temp;
struct	messagequeue *last;

printf("Dequeueing message.\n");
strcpy(buf,"");
temp = mqlist;
last = mqlist;
x=0;
while(temp != NULL)
	{
	printf("%d) h: .%s. c: .%s. %s\n",x,temp->host,host,temp->message); 
	if(strcmp(temp->host,host) == 0)
		{
		printf("Found one.\n");
		strcat(buf,temp->message);
		if(strncmp(buf,"URL: ",5) != 0)
			strcat(buf,"<BR>");
		if(temp == mqlist)
			{
			mqlist = temp->next;
			last = mqlist;
			free(temp);
			temp=last;
			}
		else
			{
			last->next = temp->next;
			free(temp);
			temp = last->next;
			}
		}
	else
		{
		last = temp;
		temp = temp->next;
		}	
	}
}

/* 
This function adds a specific host to a wait queue.  When the rccar is locked,
the waitqueue will hold a list of users waiting.  When a lock expires, the
next host on the queue receives the lock.  Feedback to the client will permit
them to start playing.
*/

int rccarwaitenqueue(char *host, struct	rccardata *rccar)
{
struct	waitqueue *temp;
struct	waitqueue *holder;

holder = malloc(sizeof *holder);
strcpy(holder->host,host);
holder->priority = 0;
holder->lastupdate = time(NULL);
time(&holder->starttime);
holder->next = NULL;
temp = rccar->wq;
if(temp == NULL)
	rccar->wq = holder;
else
	{
	// already on the queue?  They will just reassume previous position.
	if(strcmp(host,temp->host) == 0 && strcmp(host,"") != 0)
		{
		free(holder);
		return(1);
		}
	while(temp->next != NULL)
		{
		temp = temp->next;
		if(strcmp(host,temp->host) == 0 && strcmp(host,"") != 0)
			{
			free(holder);
			return(1);
			}
		}
	temp->next = holder;
	}
return(1);
}

/*
This function removes the first entry from the waitqueue and sets the lock
for the selected host.  The expiration time it set for 5 minutes after current
time.  Lock is set at the time of dequeue.
*/

int	rccarwaitdequeue(struct rccardata *rccar)
{
struct	waitqueue *temp;
int	lastupdate;

while(1)
	{
	temp = rccar->wq;
	if(temp == NULL)
		{
		rccar->lock = 0;
		return(0);
		}
	rccar->lock = 1;
	rccar->wq = rccar->wq->next;
	strcpy(rccar->lockhost,temp->host);
	rccar->lockexp = time(NULL) + 5*60;
	rccar->heartbeat = time(NULL);
	lastupdate = temp->lastupdate;
	free(temp);
	if(lastupdate > time(NULL) - 60)
		break;
	}
}


int	x10dequeue(int *insanity, int inslamp)
{
struct	x10eventqueue *temp;

//printf("Dequeuing x10.\n");
temp = x10eq;
if(temp == NULL)
	{
	return(0);
	}
x10eq = x10eq->next;
//printf("buffer: %s\n",temp->buf);
if(strncmp(temp->buf,"X10ON",5) == 0)
	{
	x10lampon(temp->buf,insanity,inslamp);
	}
else if(strncmp(temp->buf,"X10OF",5) == 0)
	{
	x10lampoff(temp->buf);
	}
free(temp);
}

int x10enqueue(char *buf, char *host)
{
struct	x10eventqueue *temp;
struct	x10eventqueue *holder;
int	hostcount;	//how many entires does this host have in the queue?

//printf("Enquing x10\n");
/*
if(bancheck)
	if(bancheckfunc(banwatchlist, host,200))
		return(0);
*/
hostcount=0;
holder = malloc(sizeof *holder);
strcpy(holder->buf,buf);
strcpy(holder->host,host);
holder->next = NULL;
temp = x10eq;
if(temp == NULL)
	x10eq = holder;
else
	{
	if(strcmp(buf,temp->buf) == 0) // already in the queue?
		{
		free(holder);
		return(0);
		}
	while(temp->next != NULL)
		{
		if(strcmp(buf,temp->buf) == 0) // command already in the queue?
			{
			free(holder);
			return(0);
			}
		if(strcmp(temp->host,host) == 0) // host already in queue.
			hostcount++;
		if(hostcount > 3)
			{
			free(holder);
			return(0);
			}
		temp = temp->next;
		}
	temp->next = holder;
	}
//printf("done enquing.\n");
}


/* isavailable
This function takes the starting availability at midnight and a list of 
times in minutes from midnight.  Figure out if the device is currently 
available.  0 for a time value means its the last of the list.
*/

int	isavailable(int start, long *times)
{
int	mins;
int	x,y,z;
long	t;
struct	tm	*tim;

//First, get the time now and convert it into minutes from midnight.
time(&t);
tim = localtime(&t);
mins = tim->tm_hour*60 + tim->tm_min;

// now find out what the current availability is
x = start;
for(z=0;z<5;z++)
	{
	if(times[z] == 0)
		break;
	if(times[z] <= mins)
		x = !x;
	}

return(x);
}




//int lampon(int portnum, int *status, long *counter, int *state, int *insanity, int inslamp)
int lampon(struct switchdata *sd, int *state, int *insanity, int inslamp)
{
char	s[256];
int	x;

/*
if(bancheck)
	if(bancheckfunc(banwatchlist, host,200))
		return(0);
*/
x = 1 << sd->port;
// First, check to see that the lamp is currently off.
if(sd->status == 1 && sd->switchtype != 1)
	{
	printf("Lamp is currently off.\n");
	return(-1); 
	}
// Next, check to see if the lamp is currently available
if(!isavailable(sd->startstatus,sd->times))
	{
	printf("Lamp is unavailable.\n");
	return(-1);
	}

// Next, see if the lamp is used up yet
if(sd->timeused > sd->maxsecs)
	{
	printf("Time used up.  timeused: %d\n",sd->timeused);
	return(-1);
	}
// Next see if this lamp is relayed
if(sd->control == 3)
	{
	printf("I GOT HERE!!!!!!!!\n");
	// we send an ON request to the slave.
	sprintf(s,"%s ON 3 spaz\n",sd->relayname);
	printf("s = .%s.\n",s);
	if(sd->relaythread >= 0)
		write(slavedata[sd->relaythread].msgsock,s,strlen(s)+1); 	
	else
		write(myslave.msgsock,s,strlen(s)+1); 	
	sd->status=1;
	sd->counter = time(NULL);
	*insanity = *insanity + (inslamp);
	printf("insanity: %d. buffer: .%s.\n",*insanity,s);
	return(1);
	}
if(ioperm(sd->switchbase,1,1))
	fprintf(stderr, "Couldn't get the port at %x\n", sd->switchbase), exit(1);
else
	{
	sd->status=1;
	sd->counter = time(NULL);
	sprintf(s,"ON");
	*state = *state | x;
   	outb(*state, sd->switchbase);
	*insanity = *insanity + (inslamp*3);
	printf("insanity: %d\n",*insanity);
	if(sd->switchtype == 1)
		{
		usleep(25000);
		if(ioperm(sd->switchbase,1,1))
			fprintf(stderr, "Couldn't get the port at %x\n", sd->switchbase), exit(1);
		sd->status=0;
		sprintf(s,"OFF");
		*state = *state & (255-x);
   		outb(*state, sd->switchbase);
		
		// Increment the timeused variable
		}
	}
}


x10lampon(char *buf, int *insanity, int inslamp)
{	
int unit;
int	fd;
unsigned short	buildunit;
char	buffer[10];
char	house;
int	housenum;
int	timelimit;
time_t	lt;
struct	tm	*tim;
char	s[256];


printf("turning on X10 unit.\n");
/* Let there be LIGHT!!! */
house = buf[6];
housenum = house - 'A';
unit = atoi(&buf[7]);
lt = time(NULL);
tim = localtime(&lt);

// First, check to see that the lamp is currently off.
if(X10mods[housenum][unit].state == 1)
	return(-1); 

// Next, check to see if the lamp is currently available
if(!isavailable(X10mods[housenum][unit].startstatus,X10mods[housenum][unit].times))
	return(-1);

// Next, see if the lamp is used up yet
if(X10mods[housenum][unit].timeused > X10mods[housenum][unit].maxsecs)
	return(-1);

if(1) 
	{
	X10mods[housenum][unit].counter = time(NULL);
	timelimit=atoi(&buf[10]);
	printf("Timelimit: %d\n",timelimit);
	if(timelimit == -1)
		{
		if(X10mods[housenum][unit].state == 0)
			X10mods[housenum][unit].limit = 60;
		}
	else if(timelimit == -2)
		{
		X10mods[housenum][unit].limit = 1;
		}
	else
		{
		X10mods[housenum][unit].limit = timelimit;
		if(!X10mods[housenum][unit].state)
			{
			if(X10mods[housenum][unit].type == 1)
				{
				fd = open(brport,O_RDONLY | O_NONBLOCK);
				buildunit = (housenum << 4) | (unit-1);
				x10_br_out(fd,buildunit,0);
				close(fd);
				}
//				sprintf(s,"%s %c%d on",brpath,house,unit);
			if(X10mods[housenum][unit].type == 2)
				{
				printf("writing to port cm11a.\n");
				fd = x10_open(brport);
				buffer[0]=HEADER_DEFAULT;
				buffer[1]=(housecode_table[housenum] << 4) | (devicecode_table[unit-1]);
				if(!x10_write_message(fd,&buffer,2))
					printf("Bad write.\n");
				buffer[0]=HEADER_DEFAULT | HEADER_FUNCTION;
				buffer[1]=(housecode_table[housenum] << 4) | COMMAND_ON;
				if(!x10_write_message(fd,&buffer,2))
					printf("Bad write\n");
				printf("done.\n");
				close(fd);
				}
//				sprintf(s,"%s %c%d:ON",ppowerpath,house-'A'+'a',unit);
//			system(s);
			}
		*insanity = (inslamp) + *insanity;
		printf("insanity: %d\n",insanity);
		}
	X10mods[housenum][unit].state = 1;
	}
}

x10lampon2(int house, int unit, int *insanity, int inslamp)
{
char	buf[256];
int	x,y,z;

sprintf(buf,"XXXXXX%c%d",house + 'A', unit);
x10lampon(buf,insanity,inslamp);
}

x10lampoff(char *buf)
{
int unit;
int	fd;
char	buffer[10];
unsigned short buildunit;
char	house;
int	housenum;
int timelimit;
char	s[256];

printf("turning off X10 unit.\n");
// turn it off
house = buf[6];
housenum = house - 'A';
unit = atoi(&buf[7]);
timelimit=atoi(&buf[10]);
X10mods[housenum][unit].limit = 0;
if(timelimit != -1 && X10mods[housenum][unit].state)
	{
	if(X10mods[housenum][unit].type == 1)
		{
		fd = open(brport,O_RDONLY | O_NONBLOCK);
		buildunit = (housenum << 4) | (unit-1);
		x10_br_out(fd,buildunit,1);
		close(fd);
		}
//		sprintf(s,"%s %c%d off",brpath,house,unit);
	if(X10mods[housenum][unit].type == 2)
		{
		printf("writing to port cm11a.\n");
		fd = x10_open(brport);
		buffer[0]=HEADER_DEFAULT;
		buffer[1]=(housecode_table[housenum] << 4) | (devicecode_table[unit-1]);
		if(!x10_write_message(fd,&buffer,2))
			printf("Bad write.\n");
		buffer[0]=HEADER_DEFAULT | HEADER_FUNCTION;
		buffer[1]=(housecode_table[housenum] << 4) | COMMAND_OFF;
		if(!x10_write_message(fd,&buffer,2))
			printf("Bad write\n");
		printf("done.\n");
		close(fd);
		}

	//	sprintf(s,"%s %c%d:OFF",ppowerpath,house-'A'+'a',unit);
	X10mods[housenum][unit].state = 0;
//	system(s);
	}
}

x10lampoff2(int house, int unit)
{
char	buf[256];
int	x,y,z;

sprintf(buf,"XXXXXX%c%d",house + 'A', unit);
x10lampoff(buf);
}


int lampoff(struct switchdata *sd, int *state)
{
char	s[256];
int	x;
long	temp;

/*
if(bancheck)
	if(bancheckfunc(banwatchlist, host,200))
		return(0);
*/
x = 1 << sd->port;
// First, make sure it's actually on, or we do nothing.
if (sd->status == 0 && sd->switchtype != 1)
	return(-1);
if(sd->control == 3)
	{
	// we send an OFF request to the slave.
	sprintf(s,"%s OFF 0 spaz\n",sd->relayname);
	if(sd->relaythread >= 0)
		write(slavedata[sd->relaythread].msgsock,s,strlen(s)+1); 	
	else
		write(myslave.msgsock,s,strlen(s)+1); 	
	sd->status=0;
	sd->counter = time(NULL);
	temp = time(NULL) - sd->counter;
	sd->timeused = sd->timeused + temp;	
	return(1);
	}
if (ioperm(sd->switchbase,1,1))
	fprintf(stderr, "Couldn't get the port at %x\n", sd->switchbase), exit(1);
else
	{
	if(sd->switchtype == 1)
		{
		int	xxx;
		sd->status=1;
		sd->counter = time(NULL);
		sprintf(s,"ON");
		*state = *state | x;
   		outb(*state, sd->switchbase);
		usleep(25000);
		}
	sd->status=0;
	sprintf(s,"OFF");
	*state = *state & (255-x);
   	outb(*state, sd->switchbase);
	
	// Increment the timeused variable
	temp = time(NULL) - sd->counter;
	sd->timeused = sd->timeused + temp;	

	}
}

/*  pan
This function will pan a specified motor one step to the left or right
depending on the value of direction.  1 = left, otherwise right.  
The override variable if 1 will not do bounds checking (for calibration)
*/

int	pan(int direction,struct motordata *md, int *state,int override)
{
int	hold;
char	s[2000];
int	x,y,z;

x=0;
printf("Current state: %d\n",*state);
if(md->control == 3)
	{
	printf("I GOT HERE!!!!!!!!\n");
	// we send an ON request to the slave.
	if(direction)
		sprintf(s,"%s PANLEFT %d spaz\n",md->relayname,1);
	else
		sprintf(s,"%s PANRIGHT %d spaz\n",md->relayname,1);
	printf("We're sending this data: .%s.\n",s);
	printf("relaythread:  %d\n",md->relaythread);
	if(md->relaythread >= 0)
		{
		printf("sock= %d\n",slavedata[md->relaythread].msgsock);
		write(slavedata[md->relaythread].msgsock,s,strlen(s)+1); 	
		}
	else
		write(myslave.msgsock,s,strlen(s)+1); 	
	return(1);
	}
for(z=0;z<md->numsequence;z++)
	{
	x = x + md->sequence[z];
	}
printf("X: %d\n",x);

printf("Override = %d, direction = %d, currentstep = %d\n",override,direction,md->currentstep);

if(!override && direction == 1 && md->currentstep < 0) // Left barrier
	return(0);
if(!override && direction == 0 && md->currentstep > 70) // Right barrier
	return(0);
printf("Still here.\n");
if (ioperm(md->motorbase,1,1))
	fprintf(stderr, "Couldn't get the port at %x\n", md->motorbase), exit(1);
else
	{
	hold = md->lastcontact;
	if(direction == 1) // Left
		{
		md->lastcontact++;
		if(md->lastcontact > 3)
			md->lastcontact=0;
		md->currentstep--;
		}
	else
		{
		md->lastcontact--;
		if(md->lastcontact < 0)
			md->lastcontact=3;
		md->currentstep++;
		}
	*state = *state & 255-x;
	*state = *state | md->sequence[md->lastcontact] | md->sequence[hold];
	printf("New state: %d\n",*state);
   	outb(*state, md->motorbase);
	}
}

int	zoom(int direction,struct motordata *md, int *state,int distance)
{
int	hold;
int	x,y,z;
char	s[256];

x=0;
printf("Current state: %d\n",*state);

printf("Still here. name: .%s. control=%d\n",md->name,md->control);
if(md->control == 3)
	{
	printf("I GOT HERE!!!!!!!! socket: %d\n",slavedata[md->relaythread].msgsock);
	// we send an ON request to the slave.
	if(direction)
		sprintf(s,"%s ZOOMOUT %d spaz\n",md->relayname,distance);
	else
		sprintf(s,"%s ZOOMIN %d spaz\n",md->relayname,distance);
	if(md->relaythread >= 0)
		write(slavedata[md->relaythread].msgsock,s,strlen(s)+1); 	
	else
		write(myslave.msgsock,s,strlen(s)+1); 	
	return(1);
	}
if(direction)
	x = md->zoomoutpin;
else
	x = md->zoominpin;
*state = *state | x;
if (ioperm(md->motorbase,1,1))
	fprintf(stderr, "Couldn't get the port at %x\n", md->motorbase), exit(1);
else
   	outb(*state, md->motorbase);
usleep(250000*distance);
*state = *state & 255-x;
if (ioperm(md->motorbase,1,1))
	fprintf(stderr, "Couldn't get the port at %x\n", md->motorbase), exit(1);
else
   	outb(*state, md->motorbase);
printf("New state: %d\n",*state);
}

int	zoomcalibrate(struct motordata *md)
{
}

int	zoomto(int position, struct motordata *md, int *state)
{
}

forwardon(struct rccardata *rccar, int *state2)
{
printf("Forward on.\n");
if (ioperm(rccar->carbase,1,1))
	fprintf(stderr, "Couldn't get the port at %x\n", rccar->carbase), exit(1);
else
	{
	*state2 = *state2 | rccar->forwardport;
	if(!rccar->wstatus)
		*state2 = *state2 | rccar->commonport;
	printf("State2 at TOF is: %d.\n",*state2);
	outb(*state2, rccar->carbase);
	}
}

forwardoff(struct rccardata *rccar, int *state2)
{
printf("Forward off.\n");
if (ioperm(rccar->carbase,1,1))
	fprintf(stderr, "Couldn't get the port at %x\n", rccar->carbase), exit(1);
else 
	{
	*state2 = *state2 & (255 - rccar->forwardport);
	if(!rccar->wstatus)
		*state2 = *state2 & (255 - rccar->commonport);
	printf("State2 at TOF is: %d.\n",*state2);
	outb(*state2, rccar->carbase);
	}
}

backwardon(struct rccardata *rccar, int *state2)
{
printf("Backward on.\n");
if (ioperm(rccar->carbase,1,1))
	fprintf(stderr, "Couldn't get the port at %x\n", rccar->carbase), exit(1);
else
	{
	*state2 = *state2 | rccar->backwardport;
	if(!rccar->wstatus)
		*state2 = *state2 | rccar->commonport;
	printf("State2 at TOB is: %d.\n",*state2);
	outb(*state2, rccar->carbase);
	}
}

backwardoff(struct rccardata *rccar, int *state2)
{
printf("Backward off.\n");
if (ioperm(rccar->carbase,1,1))
	fprintf(stderr, "Couldn't get the port at %x\n", rccar->carbase), exit(1);
else
	{
	*state2 = *state2 & (255 - rccar->backwardport);
	if(!rccar->wstatus)
		*state2 = *state2 & (255 - rccar->commonport);
	printf("State2 at TOB is: %d.\n",*state2);
	outb(*state2, rccar->carbase);
	}
}

power(int speed, int direction, struct rccardata *rccar, int *state2)
{
printf("Power function.  Direction = %d.  speed = %d.\n",direction, speed);
if(direction == 1)
	{
	backwardoff(rccar, state2);
	forwardon(rccar,state2);
	if(speed == 0)
		usleep(rccar->nudgef);
	else
		usleep(rccar->pushf);
	forwardoff(rccar,state2);
	sleep(1); 
	}
else
	{
	forwardoff(rccar,state2);
	backwardon(rccar, state2);
	if(speed == 0)
		usleep(rccar->nudgeb);
	else
		usleep(rccar->pushb);
	backwardoff(rccar,state2);
	sleep(1); 
	}
}


turnonleft(struct rccardata *rccar, int *state2)
{
if (ioperm(rccar->carbase,1,1))
	fprintf(stderr, "Couldn't get the port at %x\n", rccar->carbase), exit(1);
else
	{
	/* Turn on left */
	*state2 = *state2 | rccar->leftport | rccar->commonport;
   	outb(*state2,rccar->carbase);
	}
}

turnoffleft(struct rccardata *rccar, int *state2)
{
if (ioperm(rccar->carbase,1,1))
	fprintf(stderr, "Couldn't get the port at %x\n", rccar->carbase), exit(1);
else
	{
	/* Turn on left */
	*state2 = *state2 & (255 - (rccar->leftport + rccar->commonport));
   	outb(*state2,rccar->carbase);
	}
}

turnonright(struct rccardata *rccar, int *state2)
{
if (ioperm(rccar->carbase,1,1))
	fprintf(stderr, "Couldn't get the port at %x\n", rccar->carbase), exit(1);
else
	{
	*state2 = *state2 | rccar->rightport | rccar->commonport;
   	outb(*state2,rccar->carbase);
	}
}

turnoffright(struct rccardata *rccar, int *state2)
{
if (ioperm(rccar->carbase,1,1))
	fprintf(stderr, "Couldn't get the port at %x\n", rccar->carbase), exit(1);
else
	{
	/* Turn on left */
	*state2 = *state2 & (255 - (rccar->rightport + rccar->commonport));
   	outb(*state2,rccar->carbase);
	}
}


int	getdirection(struct rccardata *rccar, int *state2)
{
if(*state2 & rccar->rightport)
	return(2);
if(*state2 & rccar->leftport)
	return(1);
return(0);
}

turn(int direction, struct rccardata *rccar, int *state2)
{
rccar->wcounter = time(NULL);
if(direction == 1) // left
	{
	rccar->wstatus = 1;
	turnoffright(rccar,state2);
	turnonleft(rccar,state2);
	}
else if(direction == 2) // right
	{
	rccar->wstatus = 1;
	turnoffleft(rccar,state2);
	turnonright(rccar,state2);
	}
else // center
	{
	rccar->wstatus = 0;
	turnoffleft(rccar,state2);
	turnoffright(rccar,state2);
	}
}


int	powerturn(int	powerdirection, int movedirection, int speed, struct rccardata *rccar)
{
int	savedirection;

savedirection = getdirection(rccar, getstatevar(rccar->carbase));
turn(movedirection, rccar, getstatevar(rccar->carbase));
usleep(500000);
power(speed, powerdirection, rccar, getstatevar(rccar->carbase));
turn(savedirection, rccar, getstatevar(rccar->carbase));


}

int	sockinit(int port)
{
struct	sockaddr_in server;
struct	sockaddr_in client;
int	sock;
int	length;

sock = socket(AF_INET, SOCK_STREAM, 0);
printf("sock value is: %d\n",sock);
if(sock < 0)
	{
	perror("Opening steram socket");
	exit(1);
	}
server.sin_family = AF_INET;
server.sin_addr.s_addr = INADDR_ANY;
server.sin_port = htons((u_short )port);
if (bind(sock, &server, sizeof(server)))
	{
	perror("binding stream socket");
	exit(1);
	}
length = sizeof (server);
if(getsockname(sock, &server, &length))
	{
	perror("getting socket name");
	exit(1);
	}
printf("Socket has port #%d\n",ntohs(server.sin_port));
listen(sock, 5);
return(sock);
}

/*
int	getconnection(int sock, struct sklink **sk)
{
int	rval;
struct	sockaddr_in client;
fd_set	rfds;
struct	timeval	tv;
int	retval;
int	size;
int	msgsock;

size=sizeof (client);

FD_ZERO(&rfds);
FD_SET(sock,&rfds);
tv.tv_sec=0;
tv.tv_usec=1;
retval = select(11, &rfds, NULL, NULL, &tv);
if(retval)
		{
		perror("accept");
		return(-1);
		}
	else 
		{
		addsocketlink(msgsock,sk);
		return(1);
		}
	}
else
	return(-1);
}
*/


int	getcommand(int msgsock, char *buf, char *name, char *command, char *value, char *host,char *message)
{
int	rval;
int	x,y,z;
for(z=0;z<1024;z++)
	buf[z] = 0;
if((rval = read(msgsock, buf, 1460)) < 0)
	{
	perror("Reading stream message");
	close(msgsock);
	return(-1);
	}
y = gettoken(0,buf,name);
y = gettoken(y,buf,command);
y = gettoken(y,buf,value);
y = gettoken(y,buf,host);
if(strlen(buf) > y+1)
	strcpy(message,&buf[y+1]);
else
	strcpy(message,"");
printf("rval is %d.\n",rval);
printf("buf is: %s.\n",buf);
return(rval);
}

int	readcommand(int msgsock,char *commandsbuf)
{
int	rval;
int	x,y,z;
char	buf[2000];
printf("Reading command.  Current commandsbuf = .%s.\n",commandsbuf);
for(z=0;z<1500;z++)
	buf[z] = 0;
if((rval = read(msgsock, buf, 1460)) < 0)
	{
	perror("Reading stream message");
	close(msgsock);
	return(-1);
	}
stripnulls(buf,rval);
strcat(commandsbuf,buf);
printf("Read %d characters.  new commandsbuf=.%s.\n",rval,commandsbuf);
return(rval);

}


int	newgetcommand(char *commandsbuf, char *buf, char *name, char *command, char *value, char *host,char *message)
{
int	rval;
int	x,y,z;
char	mybuf[30000];

printf("current cb[0] = %d  commandbuf: .%s.\n",commandsbuf[0],commandsbuf);
if(commandsbuf[0] == 0)
	return(-1);
for(x=0;commandsbuf[x] != 0 && commandsbuf[x] != '\n';x++);
if(commandsbuf[x] == 0)
	{
	return(-1);
	}
strncpy(buf,commandsbuf,x);
buf[x] = 0;
if(commandsbuf[x] == 0)
	{
	strcpy(commandsbuf,"");
	}
else
	{
	strcpy(mybuf,&commandsbuf[x+1]);
	strcpy(commandsbuf,mybuf);
	}
rval = x;
y = gettoken(0,buf,name);
y = gettoken(y,buf,command);
y = gettoken(y,buf,value);
y = gettoken(y,buf,host);
if(strlen(buf) > y+1)
	strcpy(message,&buf[y+1]);
else
	strcpy(message,"");
printf("rval is %d.\n",rval);
printf("buf is: %s.\n",buf);
printf("new commandsbuf= .%s.\n",commandsbuf);
return(1);
}

initbasic()
{
listenport = 5201;
bancheck = 0;
togglemax = 500;
banhours = 1;
bandeduct = 2;
kwhcost = 0.07;
strcpy(clientname,"DEF");
slaved=0;
strcpy(brport,"/dev/ttyS0");
strcpy(ppowerpath,"/usr/local/bin/ppower");
locks[0] = 60;
locks[1] = 120;
locks[2] = 300;
locks[3] = 600;
locks[4] = 1200;
}

rccarinit(struct rccardata *rccar)
{
int	x,y,z;

rccar->commonport=16;
rccar->forwardport=2;
rccar->backwardport=1;
rccar->type = 0;
rccar->leftport = 8;
rccar->rightport = 4;
rccar->carbase = base2;
rccar->nudgef = SLOW;
rccar->nudgeb = SLOW;
rccar->pushf = MEDIUM;
rccar->pushb = FAST;
rccar->wstatus = 0;
rccar->wcounter = 0;
rccar->eq = NULL;
rccar->wq = NULL;
rccar->lock = 0;
rccar->lockexp = 0;
}

initswitch(struct switchdata *sd)
{
int	x,y,z;
sd->status=0;
sd->available=1;
sd->port = 0;
sd->switchbase = base;
sd->control = 1;
sd->switchtype=0;
sd->counter = 0;
sd->watts = 60;
sd->timeused = 0;
sd->timeout = 3*60;
sd->startstatus=1;
sd->stateposition = 0;
for(z=0;z<5;z++)
	sd->times[z] = 0;
sd->maxsecs = 2000*60;
strcpy(sd->name,"");
strcpy(sd->relayhost,"127.0.0.1");
sd->relayport = 5201;
sd->wq = NULL;
sd->relaythread=-2;
sd->icon = 0;
sd->bored = 0;
strcpy(sd->relayname,"");
strcpy(sd->camname[0],"");
}

motorinit(struct motordata *md)
{
int	x,y,z;
for(z=0;z<4;z++)
	md->sequence[z] = 1 << z;
md->numsequence = 4;
md->motorbase = base;
md->currentstep = 25;
md->lastcontact = -1;
md->zoomsteptime = 250000;
md->zoomposition = 0;
md->control = 1;
md->available=1;
strcpy(md->name,"");
strcpy(md->relayhost,"127.0.0.1");
md->relayport = 0;
md->wq = NULL;
strcpy(md->relayname,"");
md->relaythread = -2;
}

addswitch(struct switchdata sd)
{
struct	sdlink *temp;
struct	sdlink *temp2;

printf("Adding switch. sd.name=%s sd.label=%s sd.relayname\n",sd.name,sd.label,sd.relayname);
temp = malloc(sizeof *temp);
memcpy(&(temp->sd),&sd,sizeof sd);
temp->next = NULL;
if(switches == NULL)
	switches = temp;
else
	{
	temp2 = switches;
	while(temp2->next != NULL)
		temp2 = temp2->next; 
	temp2->next = temp;
	}
}

addrccar(struct rccardata rc)
{
struct	rclink *temp;
struct	rclink *temp2;

printf("Adding rccar. name=%s label=%s relayname\n",rc.name,rc.label,rc.relayname);
temp = malloc(sizeof *temp);
memcpy(&(temp->rc),&rc,sizeof rc);
temp->next = NULL;
if(rccars == NULL)
	rccars = temp;
else
	{
	temp2 = rccars;
	while(temp2->next != NULL)
		temp2 = temp2->next; 
	temp2->next = temp;
	}
}


/*

addsocketlink(int msgsock,struct sklink **sk)
{
struct	sklink *temp;
struct	sklink *temp2;
struct	sklink *socketlink;
long	t;

socketlink = *sk;
printf("Adding socket. \n");
temp = malloc(sizeof *temp);
temp->msgsock = msgsock;
time(&t);
temp->connecttime = t;
temp->next = NULL;
if(socketlink == NULL)
	{
	socketlink = temp;
	*sk = socketlink;
	}
else
	{
	temp2 = socketlink;
	while(temp2->next != NULL)
		temp2 = temp2->next; 
	temp2->next = temp;
	}
}
*/

deleteswitch(char *name)
{
struct	sdlink *temp;
struct	sdlink *temp2;

printf("In Delete Switches. name=%s\n",name);
if(switches == NULL)
	{
	printf("We should never see this.\n");
	return(-1);
	}

temp = NULL;
temp = switches;

if(strcmp(switches->sd.name,name) == 0)
	{
	temp = switches;
	switches=temp->next;
	free(temp);
	printf("Done.\n");
	return(1);
	}	
while(temp->next != NULL)
	{
	if(strcmp(temp->next->sd.name,name) == 0)
		{
		temp2 = temp->next;
		temp->next = temp2->next;
		free(temp2);
		printf("Done.\n");
		return(1);	
		}
	temp = temp->next;
	}
printf("Done.  No switches removed.\n");
}

deleterccar(char *name)
{
struct	rclink *temp;
struct	rclink *temp2;

if(rccars == NULL)
	return(-1);

temp = NULL;
temp = rccars; 

if(strcmp(rccars->rc.name,name) == 0)
	{
	temp = rccars;
	rccars=temp->next;
	free(temp);
	return(1);
	}	
while(temp->next != NULL)
	{
	if(strcmp(temp->next->rc.name,name) == 0)
		{
		temp2 = temp->next;
		temp->next = temp2->next;
		free(temp2);
		return(1);	
		}
	temp = temp->next;
	}
}

/*
int	deletesocket(int msgsock,int mode,struct sklink **sk)
{
struct	sklink *temp;
struct	sklink *temp2;
struct	sklink *socketlink;

socketlink = *sk;

printf("deletesocket function.\n");
if(socketlink == NULL)
	return(-1);

temp = NULL;
temp = socketlink;

if(socketlink->msgsock == msgsock)
	{
	temp = socketlink;
	socketlink=temp->next;
	*sk = socketlink;
	free(temp);
	printf("Leaving deletesocket\n");
	return(1);
	}	
while(temp->next != NULL)
	{
	if(temp->next->msgsock == msgsock)
		{
		temp2 = temp->next;
		temp->next = temp2->next;
		free(temp2);
		printf("Leaving deletesocket\n");
		return(1);	
		}
	else
		temp=temp->next;
	}
}
*/

/*
int	scansockets(struct sklink **sk,char *host)
{
int	x,y,z;
long	t;
struct	sklink *temp;
struct	sklink *temp2;
struct	sklink *socketlink;
int	msgsock;
fd_set	rfds;
struct	timeval	tv;
int	retval;

socketlink = *sk;
time(&t);
temp = socketlink;
while(temp != NULL)
	{
	printf("Scanning msgsock %d\n",temp->msgsock);
	if(temp->connecttime + 10 < t)
		{
		close(temp->msgsock);
		deletesocket(temp->msgsock,1,&socketlink);
		return(0);
		}
	FD_ZERO(&rfds);
	FD_SET(temp->msgsock,&rfds);
		
	tv.tv_sec=0;
	tv.tv_usec=1;	
	retval = select(11, &rfds, NULL, NULL, &tv);
	if(retval)
		{
		strncpy(host,temp->host,39);
		host[39] = 0;
		return(temp->msgsock);
		}
	else
		temp = temp->next;
	}
return(-1);
}
*/

deletemotor(char *name)
{
struct	mdlink *temp;
struct	mdlink *temp2;

if(motors == NULL)
	return(-1);
temp = motors;

if(strcmp(motors->md.name,name) == 0)
	{
	temp = motors;
	motors=temp->next;
	free(temp);
	return(1);
	}	
while(temp->next != NULL)
	{
	if(strcmp(temp->next->md.name,name) == 0)
		{
		temp2 = temp->next;
		temp->next = temp2->next;
		free(temp2);
		return(1);	
		}
	temp = temp->next;
	}
}

purge_switches(int threadnum)
{
int	x,y,z;
char	name[20];
struct	sdlink *temp;
struct	mdlink *temp2;

printf("purging switches. threadnum == %d\n",threadnum);
temp=switches;
while(temp != NULL)
	{
	if(temp->sd.relaythread == threadnum)
		{
		strcpy(name,temp->sd.name);
		temp = temp->next;	
		deleteswitch(name);
		}
	else
		temp = temp->next;
	}
temp2=motors;
while(temp2 != NULL)
	{
	if(temp2->md.relaythread == threadnum)
		{
		strcpy(name,temp2->md.name);
		temp2 = temp2->next;	
		deletemotor(name);
		}
	else
		temp2 = temp2->next;
	}
printf("Done.\n");
}


testswitches()
{
int	x;
struct	sdlink *test;

test = switches;
while(test != NULL)
	{
	lampon(&test->sd,getstatevar(test->sd.switchbase),&x,0);
	sleep(2);	
	lampoff(&test->sd,getstatevar(test->sd.switchbase));
	test=test->next;
	}
}


addmotor(struct motordata md)
{
struct	mdlink *temp;
struct	mdlink *temp2;

temp = malloc(sizeof *temp);
memcpy(&(temp->md),&md,sizeof md);
temp->next = NULL;
if(motors == NULL)
	motors = temp;
else
	{
	temp2 = motors;
	while(temp2->next != NULL)
		temp2 = temp2->next; 
	temp2->next = temp;
	}
}

int	x10init(struct	X10data  *x10d)
{
int	x,y,z;

x10d->state = 0;
x10d->available=1;
x10d->limit = 0;
x10d->counter = 0;
x10d->timeused = 0;
x10d->watts = 60;
x10d->remotetimeout = 3*60;
x10d->localtimeout = 60*60;
x10d->startstatus = 1;
x10d->type = 1;
x10d->stateposition = 0;
for(z=0;z<5;z++)
	x10d->times[z] = 0;
x10d->maxsecs = 2000 * 60;
x10d->control = 1;
strcpy(x10d->relayhost,"");
x10d->relayport = 0;
x10d->relayhouse = 0;
x10d->wq = NULL;
strcpy(x10d->label,"");
strcpy(x10d->relayname,"");
x10d->relaythread=0;
x10d->icon=0;
x10d->bored=0;
}

printconfig()
{
int	x,y,z;
struct	sdlink *temp;

printf("This is a display of the data currently in memory.  \n\n");

// First the switches.

temp = switches;
while(temp != NULL)
	{
	printf("[Switch]\n");
	printf("Name: %s\n",temp->sd.name);
	printf("Port: %d\n",temp->sd.port);
	printf("Status: %d\n",temp->sd.status);
	printf("Base: %x\n",temp->sd.switchbase);
	printf("Control: %d\n",temp->sd.control);
	printf("Counter: %d\n",temp->sd.counter);
	printf("Stateposition: %d\n",temp->sd.stateposition);
	printf("Timeout: %d\n",temp->sd.timeout);
	printf("maxsecs: %d\n",temp->sd.maxsecs);
	temp = temp->next;
	}
for(y=0;y<17;y++)
	for(x=0;x<17;x++)
		{
		if(X10mods[y][x].label[0] != 0)
			printf("house: %d unit: %d  available: %d  label: %s\n",y,x,X10mods[y][x].available,X10mods[y][x].label);
		}
}


int	create_config_file()
{
int	x,y,z;
char	s[2000];
FILE	*output;

output = fopen("~/lampserver.xml","w+");
fprintf(output,"<lampmaster2000>\n");
fprintf(output,"	<basic>\n");

x = 5201;
fprintf(output,"		<listenport>%d</listenport>\n",x);

x = 500;
fprintf(output,"		<togglemax>%d</togglemax>\n",x);

x = 0;
fprintf(output,"		<bancheck>%d</bancheck>\n",x);

x = 48;
fprintf(output,"		<banhours>%d</banhours>\n",x);

x = 60;
fprintf(output,"		<bandeduct>%d</bandeduct>\n",x);

strcpy(s,"64.246.28.98");
fprintf(output,"		<slave>%s</slave>\n",s);

strcpy(s,"/dev/ttyS1"); 
fprintf(output,"		<ttyport>%s</ttyport>\n",s);


fprintf(output,"	</basic>\n");
fprintf(output,"	<switches>\n");

printf("Do you wish to add any switches? [y/(n)] ");
gets(s);
while(s[0] == 'y' || s[0] == 'Y')
	{
	fprintf(output,"		<item>\n");
	printf("Enter the variable name for this switch: ");
	gets(s);
	if(s[0] != 0)
		fprintf(output,"			<name>%s</name>\n",s);
	else
		fprintf(output,"			<name>Test</name>\n");
	
	printf("Enter the label for this switch (the name that shows up on the site: ");
	gets(s);
	if(s[0] != 0)
		fprintf(output,"			<label>%s</label>\n",s);
	else
		fprintf(output,"			<label>Test</label>\n");
		
	printf("Enter the data pin number (0): ");
	gets(s);
	x = atoi(s);
	if(s[0] != 0)
		fprintf(output,"			<pin>%d</pin>\n",x);
	else
		fprintf(output,"			<pin>0</pin>\n");


	printf("Enter the parallel port address (0x378): ");
	gets(s);
	x = atoi(s);
	if(s[0] != 0)
		fprintf(output,"			<base>%x</base>\n",x);
	else
		fprintf(output,"			<base>0x378</base>\n");

	printf("Enter the number of seconds to wait before turning off the switch (180): ");
	gets(s);
	x = atoi(s);
	if(s[0] != 0)
		fprintf(output,"			<timeout>%d</timeout>\n",x);
	else
		fprintf(output,"			<timeout>180</timeout>\n");

	printf("Control mode: (0) Relay elsewhere, (1) local control (1): ");
	gets(s);
	x = atoi(s);
	if(s[0] != 0)
		fprintf(output,"			<control>%d</control>\n",x);
	else
		fprintf(output,"			<control>1</control>\n");

	printf("Is this switch active at midnight? (1): ");
	gets(s);
	x = atoi(s);
	if(s[0] != 0)
		fprintf(output,"			<start>%d</start>\n",x);
	else
		fprintf(output,"			<start>1</start>\n");


	printf("Enter a state toggle time or enter for no more times (00:00): ");
	gets(s);
	x = 0;
	while(s[0] != 0)
		{
		if(x == 0)
			fprintf(output,"			<times>\n");
		fprintf(output,"				<item>%s</item>\n",s);
		printf("Enter a state toggle time or enter for no more times (00:00): ");
		gets(s);
		x++;
		}
	if(x > 0)
		fprintf(output,"			</times>\n");
		

	fprintf(output,"		</item\n");	

	
	printf("Do you wish to add any more switches? [y/(n)] ");
	gets(s);
	}

fprintf(output,"	</switches>\n");
fprintf(output,"	<x10>\n");
printf("Do you wish to add any switches? [y/(n)] ");
gets(s);
while(s[0] == 'y' || s[0] == 'Y')
	{
	fprintf(output,"		<item>\n");
	
	printf("Enter the label for this device: ");
	gets(s);
	if(s[0] != 0)
		fprintf(output,"			<label>%s</label>\n",s);
	else
		fprintf(output,"			<label>Test</label>\n");
	


	fprintf(output,"		</item>\n");
	}

fprintf(output,"	</x10>\n");
fprintf(output,"	<motors>\n");

fprintf(output,"	</motors>\n");
fprintf(output,"	<rccars>\n");

fprintf(output,"	</rccars>\n");


fprintf(output,"</lampmaster2000>\n");
fclose(output);
}


readconfig()
{
FILE	*in;
int	x,y,z;
int	mode;
int	lastmode;
int	housemode;
int	unitmode;
int	times;
int	foundfile;
char	line[2000];
char	token[256];
char	value[256];
char	s[256];

struct	switchdata	sd;
struct	X10data		x10d;
struct	motordata	md;
struct	rccardata	rc;

switches = NULL;
motors = NULL;
rccars = NULL;
x10eq = NULL;
camcounts= NULL;
initswitch(&sd);
motorinit(&md);
x10init(&x10d);
rccarinit(&rc);
lastmode = 0;
mode=0;
housemode=-1;
unitmode = -1;
times=0;
for(z=0;z<17;z++)
	for(y=0;y<17;y++)
		{
		x10init(&X10mods[z][y]);
		if(z == 0)
			X10mods[z][y].stateposition = y+1;
		}
foundfile=0;
in = fopen("lampserver.cfg","r");
if(in != NULL)
	foundfile=1;
if(!foundfile)
	{
	in = fopen("~/lampserver.cfg","r");
	if(in != NULL)
		foundfile=1;
	}
if(!foundfile)
	{
	printf("The lampserver.cfg file does not exist.  Starting interactive creation.\n");
	create_config_file();
	in = fopen("~/lampserver.cfg","r");
	}
while (!feof(in))
	{
	// read a line from the config file.
	line[0] = 0;
	fgets(line, 1990, in);
	line[1990] = 0;
//	printf("Line: %s\n",line);	
	// parse the line for a comment ; and null the rest of the line.	
	x = strlen(line);
	for(z=0;z<x;z++)
		{
		if(line[z] == ';' || line[z] == '\n')
			{
			line[z] = 0;
			break;
			}
		}
	// get the first token in the line
	x = gettoken(0,line,token);
	// get the second token in the line
	y = gettoken(x,line,value);
	if(strcmp(token,"") == 0)
		continue;
	if(strcmp(token,"[basic]") == 0)
		{
		lastmode = mode;
		initbasic();
		mode = 1;
		}
	if(strcmp(token,"[switch]") == 0)
		{
		lastmode = mode;
		mode = 2;
		times = 0;
		}
	if(strcmp(token,"[x10]") == 0)
		{
		lastmode = mode;
		mode = 3;
		times = 0;
		}
	if(strcmp(token,"[motor]") == 0)
		{
		lastmode = mode;
		mode = 4;
		motorinit(&md);
		times = 0;
		}
	if(strcmp(token,"[rccar]") == 0)
		{
		lastmode = mode;
		mode = 5;
		rccarinit(&rc);
		times = 0;
		}
	if(lastmode)
		{
		if(lastmode == 2)
			{
			addswitch(sd);
			initswitch(&sd);
			}	
		if(lastmode == 4)
			{
			addmotor(md);
			motorinit(&md);
			}
		lastmode = 0;
		}
	if(mode == 1)
		{
		if(strcmp(token,"port") == 0)
			listenport = atoi(value);	
		if(strcmp(token,"togglemax") == 0)
			togglemax = atoi(value);	
		if(strcmp(token,"bancheck") == 0)
			bancheck = atoi(value);	
		if(strcmp(token,"banhours") == 0)
			banhours = atoi(value);	
		if(strcmp(token,"bandeduct") == 0)
			bandeduct = atoi(value);	
		if(strcmp(token,"kwhcost") == 0)
			kwhcost = atof(value);
		if(strcmp(token,"slave") == 0)
			{
			slaved=1;
			strncpy(myslave.host,value,39);
			myslave.host[39] =0;
			}
		if(strcmp(token,"brport") == 0)
			{
			strncpy(brport,value,39);
			brport[39] = 0;
			}
		if(strcmp(token,"ppowerpath") == 0)
			{
			strncpy(ppowerpath,value,39);
			ppowerpath[39] = 0;
			}
		if(strcmp(token,"clientname") == 0)
			{
			strncpy(clientname,value,6);
			clientname[6] = 0;
			}
		}
	if(mode == 2)
		{
		if(strcmp(token,"switchtype") == 0)
			sd.switchtype = atoi(value);
		if(strcmp(token,"pin") == 0)
			sd.port = atoi(value);
		if(strcmp(token,"base") == 0)
			sd.switchbase = atoi(value);
		if(strcmp(token,"timeout") == 0)
			sd.timeout = atoi(value);
		if(strcmp(token,"control") == 0)
			sd.control = atoi(value);
		if(strcmp(token,"watts") == 0)
			sd.watts = atoi(value);
		if(strcmp(token,"icon") == 0)
			sd.icon = atoi(value);
		if(strcmp(token,"bored") == 0)
			sd.bored = atoi(value);
		if(strcmp(token,"start") == 0)
			sd.startstatus = atoi(value);
		if(strcmp(token,"time") == 0)
			{
			int	hour;
			int	minute;
			long	minutes;
			hour = atoi(value);
			z=0;
			while(value[z++] != ':');
			minute = atoi(&value[z]);	
			minutes = hour*60+minute;
			sd.times[times++] = minutes;
			}
		if(strcmp(token,"maxsecs") == 0)
			sd.maxsecs = atoi(value);
		if(strcmp(token,"maxmins") == 0)
			sd.maxsecs = atoi(value) * 60;
		if(strcmp(token,"name") == 0)
			{
			strncpy(sd.name,value,19);
			sd.name[19] = 0;
			}
		if(strcmp(token,"label") == 0)
			{
			strncpy(sd.label,value,39);
			sd.label[39] = 0;
			}
		if(strcmp(token,"relayhost") == 0)
			{
			strncpy(sd.relayhost,value,29);
			sd.relayhost[29] = 0;
			}
		if(strcmp(token,"relayport") == 0)
			sd.relayport = atoi(value);
		if(strcmp(token,"camname") == 0)
			{
			strncpy(sd.camname[0],value,79);
			sd.camname[0][79] = 0;
			}
		}
	if(mode == 3)
		{
		int	c1,c2,d1,d2;
		if(strcmp(token,"start") == 0)
			x10d.startstatus = atoi(value);
		if(strcmp(token,"type") == 0)
			x10d.type = atoi(value);
		if(strcmp(token,"time") == 0)
			{
			int	hour;
			int	minute;
			long	minutes;
			hour = atoi(value);
			z=0;
			while(value[z++] != ':');
			minute = atoi(&value[z]);	
			minutes = hour*60+minute;
			x10d.times[times++] = minutes;
			}
		if(strcmp(token,"localtimeout") == 0)
			x10d.localtimeout = atoi(value);
		if(strcmp(token,"remotetimeout") == 0)
			x10d.remotetimeout = atoi(value);
		if(strcmp(token,"icon") == 0)
			x10d.icon = atoi(value);
		if(strcmp(token,"bored") == 0)
			x10d.bored = atoi(value);
		if(strcmp(token,"maxmins") == 0)
			x10d.maxsecs = atoi(value) * 60;
		if(strcmp(token,"maxsecs") == 0)
			x10d.maxsecs = atoi(value);
		if(strcmp(token,"watts") == 0)
			x10d.watts = atoi(value);
		if(strcmp(token,"relayhost") == 0)
			{
			strncpy(x10d.relayhost,value,29);
			x10d.relayhost[29] = 0;
			}
		if(strcmp(token,"relayport") == 0)
			x10d.relayport = atoi(value);
		if(strcmp(token,"control") == 0)
			x10d.control = atoi(value);
		if(strcmp(token,"house") == 0)
			{
			if(value[0] >= 'A' && value[0] <= 'P')
				housemode = value[0] - 'A';
			else if(value[0] >= 'a' && value[0] <= 'p')
				housemode = value[0] - 'a';
			else
				housemode = atoi(value);
			}
		if(strcmp(token,"unit") == 0)
			unitmode = atoi(value);
		if(strcmp(token,"label") == 0)
			{
			strncpy(x10d.label,value,39);
			x10d.label[39] = 0;
			}
		if(strcmp(token,"camname") == 0)
			{
			strncpy(x10d.camname[0],value,79);
			x10d.camname[0][79] = 0;
			}
		

		if(housemode == -1)
			{
			c1 = 0;
			c2 = 16;
			}
		else
			{
			c1 = housemode;
			c2 = housemode+1;
			}
		if(unitmode == -1)
			{
			d1 = 0;
			d2 = 0;
			}
		else
			{
			d1 = unitmode;
			d2 = unitmode+1;
			}
		for(z=c1;z<c2;z++)
			for(y=d1;y<d2;y++)
				{
				memcpy(&X10mods[z][y],&x10d, sizeof x10d);
				}
		}
	if(mode == 4)
		{
		if(strcmp(token,"name") == 0)
			strcpy(md.name,value);
		if(strcmp(token,"label") == 0)
			strcpy(md.label,value);
		if(strcmp(token,"zoominpin") == 0)
			md.zoominpin = 1 << atoi(value);
		if(strcmp(token,"zoomoutpin") == 0)
			md.zoomoutpin = 1 << atoi(value);
		if(strcmp(token,"zoomsteptime") == 0)
			md.zoomsteptime = atoi(value);
		if(strcmp(token,"sequence") == 0)
			md.sequence[times++] = 1 << atoi(value);
		
		}
/*
officerotor.currentstep=25;
officerotor.lastcontact = -1;
officerotor.sequence[0] = 4;
officerotor.sequence[1] = 8;
officerotor.sequence[2] = 2;
officerotor.sequence[3] = 16;
officerotor.numsequence = 4;
officerotor.motorbase = base;
officerotor.zoominpin = 2;
officerotor.zoomoutpin = 4;
*/
	}				
fclose(in);
if(mode == 2)
	{
	addswitch(sd);
	}	
if(mode == 4)
	{
	addmotor(md);
	}
		lastmode = 0;
}



struct	switchdata *searchswitch(char *name)
{
int	x,y,z;
struct	sdlink *temp;

temp = switches;
while(temp != NULL)
	{
	if(strcmp(temp->sd.name,name) == 0)
		return(&temp->sd);
	temp = temp->next;
	}
return(NULL);
}

struct	rccardata *searchrccar(char *name)
{
int	x,y,z;
struct	rclink *temp;

printf("Searching for rccars.\n");
temp = rccars;
while(temp != NULL)
	{
	printf("Checking .%s against .%s.\n",temp->rc.name,name);
	if(strcmp(temp->rc.name,name) == 0)
		return(&temp->rc);
	temp = temp->next;
	}
return(NULL);
}

struct	motordata *searchmotor(char *name)
{
int	x,y,z;
struct	mdlink *temp;

temp = motors;
while(temp != NULL)
	{
	printf("comparing .%s. with .%s.\n",temp->md.name, name);
	if(strcmp(temp->md.name,name) == 0)
		return(&temp->md);
	temp = temp->next;
	}
return(NULL);
}

int	searchx10(char *name,int *house, int *unit)
{
int	x,y,z;
printf("searching for X10 devices.\n");
if(strlen(name) < 2)
	return(0);
if(name[0] < 'A' || name[0] > 'P')
	return(0);
x = atoi(&name[1]);
if(x <= 0 || x > 16)
	return(0);
y = name[0] - 'A';
*house = y;
*unit = x;
printf("Found one.  y=%d x=%d\n",y,x);
return(1);
}


/* checktimeouts()
This function searches through all linked lists, checks all switches that
have timeouts, and toggles them off if the time has exceeded the max.
*/

int	checktimeouts(struct	rccardata *rccar)
{
int	x,y,z;
char	s[256];
struct	sdlink *temp;
struct	switchdata *temp2;

// first switches
//printf("In checktimeouts.\n");
deletebans(banwatchlist);
deletebans(rccarwatchlist);

temp = switches;
//printf("temp: %d\n",temp);
while(temp != NULL)
	{
//	printf("temp: %d   status: %d  time: %d   counter: %d\n",temp,temp->sd.status,time(NULL), temp->sd.counter);
	if(temp->sd.status && time(NULL) > 3*60+temp->sd.counter)
		{
//		printf("Got here.\n");
		temp2 = &(temp->sd);
		lampoff(temp2,getstatevar(temp->sd.switchbase));
		}
	temp = temp->next;
	
	}

if(rccar->wq != NULL)
	{
	printf("There's a host on the queue.  Checking.\n");
	if(rccar->lock)
		{
		printf("Car is locked.  exp: %d  time: %d\n",rccar->lockexp, time(NULL));
		if(rccar->lockexp < time(NULL) || rccar->heartbeat < time(NULL)-60)
			{
			// dequeue if expired or idle for more than 60s.
			rccarwaitdequeue(rccar);
			}
		}
	else
		{
		printf("Dequeueing.\n");
		rccarwaitdequeue(rccar);
		}
		
	}
if(rccar->lock)
	{
	printf("Car is locked.  exp: %d  time: %d\n",rccar->lockexp, time(NULL));
	if(rccar->lockexp < time(NULL) || rccar->heartbeat < time(NULL)-60)
		{
		// dequeue if expired or idle for more than 60s.
		rccarwaitdequeue(rccar);
		}
	}
//printf("Leaving checktimeouts.\n");
}

// This function is called at midnight once a day to reset all timeused
// variables.
int	resettimes(struct switchdata *osd, struct switchdata *ssd)
{
int	x,y,z;
struct	sdlink *temp;
printf("In resettimes.\n");
osd->timeused = 0;
ssd->timeused = 0;
temp = switches;
while(temp != NULL)
	{
	temp->sd.timeused = 0;
	temp = temp->next;
	}
printf("Leaving resettimes.\n");
}

/* computepower
This function goes through the database and computes the daily power 
consumption and returns an integer representing wattseconds for the day.
*/

int	computepower()
{
int	x,y,z;
int	ws; // wattseconds
float	wh; // watthours
float	f;
struct	sdlink *temp;

ws = 0;

temp = switches;
while(temp != NULL)
	{
	z = (temp->sd.timeused * temp->sd.watts); 
	ws = ws + z;
	temp = temp->next;
	}
for(z=0;z<17;z++)
	for(y=0;y<17;y++)
		{
		x = X10mods[z][y].timeused * X10mods[z][y].watts;
		ws = ws + x;
		}
return(ws);
}

int	timeremaining(struct switchdata *sd)
{
if(!isavailable(sd->startstatus,sd->times))
	{
	printf("Lamp is unavailable.\n");
	return(0);
	}

// Next, see if the lamp is used up yet
if(sd->timeused > sd->maxsecs)
	{
	printf("Time used up.  timeused: %d\n",sd->timeused);
	return(0);
	}
return(sd->maxsecs - sd->timeused);
}

getSTATE(char *s, int insanity, int *compstate, int strike)
{
int	x,y,z;
int	secondsleft;
char	tt[100];
long	lt;
struct	tm	*tim;
char	buf[2000];
char	w[200];
int	first;
struct	sdlink *temp;

first=1;
strcpy(buf,"");

temp = switches;
while(temp != NULL)
	{
	if(!first)
		{
		strcat(buf,"&");
		}
	sprintf(tt,"%s=%d",temp->sd.name,temp->sd.status);
	strcat(buf,tt);
	first=0;
	temp = temp->next;
	}

for(y=0;y<17;y++)
	for(x=0;x<17;x++)
		{
		if(X10mods[y][x].label[0] != 0)
			{
			sprintf(w,"%c%d",y+'A',x);	
			if(X10mods[y][x].available)
				{
				if(!first)
					{
					strcat(buf,"&");
					}
				first=0;
				sprintf(tt,"%s=%d",w,X10mods[y][x].state);
				strcat(buf,tt);
				}
			}
		}
if(!first)
	{
	strcat(buf,"&");
	}
sprintf(tt,"insanity=%d",insanity);
strcat(buf,tt);
sprintf(tt,"&strike=%d",strike);
strcat(buf,tt);
printf("buffer=.%s.\n",buf);
strcpy(s,buf);
}

delete_camcount(char *name)
{
struct	sdlink *temp;
struct	sdlink *temp2;

if(switches == NULL)
	return(-1);

temp = NULL;
temp = switches;

if(strcmp(switches->sd.name,name) == 0)
	{
	temp = switches;
	switches=temp->next;
	free(temp);
	return(1);
	}	
while(temp->next != NULL)
	{
	if(strcmp(temp->next->sd.name,name) == 0)
		{
		temp2 = temp->next;
		temp->next = temp2->next;
		free(temp2);
		return(1);	
		}
	}
}

int	lsgetcamcount(int	camnum, char *buf)
{
int	x,y,z;
int	count;
struct	cclink *temp;

printf("getcamcount function.\n");
count=0;
temp = camcounts;
while(temp != NULL)
	{
	printf("temp=%d\n",temp);
	if(temp->cc.cam == camnum)
		count++;
	temp = temp->next;
	}
printf("about to sprintf.\n");
sprintf(buf,"%d",count);
}

int	getlistcamhosts(int	camnum, char *buf)
{
int	x,y,z;
int	count;
char	s[2000];
struct	cclink *temp;

strcpy(buf,"");
printf("getcamcount function.\n");
count=0;
temp = camcounts;
while(temp != NULL)
	{
	printf("temp=%d\n",temp);
	if(temp->cc.cam == camnum)
		{
		sprintf(s,"%s\n",temp->cc.host);
		strcat(buf,s);
		}
	temp = temp->next;
	}
printf("about to sprintf.\n");
}

deletecamcounts()
{
struct	cclink *temp;
struct	cclink *temp2;
long	current;

//printf("Delete camcounts function.\n");
current = time(NULL);
if(camcounts == NULL)
	return(-1);

temp = NULL;
temp = camcounts;

if(camcounts->cc.lastupdate + 60 < current)
	{
	temp = camcounts;
	camcounts=temp->next;
	free(temp);
	printf("returning.\n");
	return(1);
	}	
while(temp->next != NULL)
	{
	if(temp->next->cc.lastupdate + 60 < current)
		{
		temp2 = temp->next;
		temp->next = temp2->next;
		free(temp2);
		}
	temp=temp->next;
	if(temp == NULL)
		break;
	}
//printf("Returning.\n");
}

int	update_camcount_list(int camnum,char *host)
{
int	x,y,z;
int	mode;
struct	cclink *temp;
struct	cclink *temp2;

printf("update camcounts.\n");
temp = malloc(sizeof *temp);
temp->cc.cam = camnum;
strcpy(temp->cc.host,host);
temp->cc.lastupdate = time(NULL);
temp->next = NULL;
if(camcounts == NULL)
	camcounts = temp;
else
	{
	temp2 = camcounts;
	while(temp2->next != NULL && strcmp(temp2->cc.host,host) != 0)
		temp2 = temp2->next; 
	if(strcmp(temp2->cc.host,host) == 0)
		{
		temp2->cc.lastupdate = time(NULL);
		temp2->cc.cam = camnum;
		free(temp);
		}
	else
		temp2->next = temp;
	}
printf("returning.\n");
}



newgetSTATE(char *s, int insanity, int *compstate, int strike,int camnum, char *host)
{
int	x,y,z;
int	secondsleft;
char	tt[100];
long	lt;
struct	tm	*tim;
char	buf[2000];
char	w[200];
int	first;
struct	sdlink *temp;

first=1;
strcpy(buf,"");
if(camnum >= 0)
	update_camcount_list(camnum,host);
temp = switches;
while(temp != NULL)
	{
	if(!first)
		{
		strcat(buf,"&");
		}
	sprintf(tt,"%s=%d",temp->sd.name,temp->sd.status);
	strcat(buf,tt);
	first=0;
	temp = temp->next;
	}

for(y=0;y<17;y++)
	for(x=0;x<17;x++)
		{
		if(X10mods[y][x].label[0] != 0)
			{
			sprintf(w,"%c%d",y+'A',x);	
			if(X10mods[y][x].available)
				{
				if(!first)
					{
					strcat(buf,"&");
					}
				first=0;
				sprintf(tt,"%s=%d",w,X10mods[y][x].state);
				strcat(buf,tt);
				}
			}
		}
if(!first)
	{
	strcat(buf,"&");
	}
sprintf(tt,"insanity=%d",insanity);
strcat(buf,tt);
sprintf(tt,"&strike=%d",strike);
strcat(buf,tt);
printf("buffer=.%s.\n",buf);
strcpy(s,buf);
}

oldgetSTATE(char	*s, int	insanity, int *compstate, int strike,struct switchdata sprinksd)
{
int	x,y,z;
int	secondsleft;
char	tt[100];
long	lt;
struct	tm	*tim;

printf("In state procedure.\n");
//s[12+17] = '2';
s[0] = (state & 1) + '0';
for(z=1;z<17;z++)
	{
	s[z] = X10mods[0][z].state + '0';
	}
for(z=0;z<20;z++)
	{
	s[z+17] = compstate[z] + '0';
	}
//s[12+17] = '2';
secondsleft = timeremaining(&sprinksd);
if(secondsleft > 0)
	s[12+17] = sprinksd.status+'0';
else
	s[12+17] = '2';
s[37]=(secondsleft/60)/10 + '0';
s[38]=(secondsleft/60)%10 + '0';
s[39]=(secondsleft%60)/10 + '0';
s[40]=(secondsleft%60)%10 + '0';	
sprintf(tt,"%d ",insanity);
strcpy(&s[41],tt);
sprintf(tt,"%d ",strike);
strcat(s,tt);
printf("s len= %d\n",strlen(s));
}


waitqueuelist(char *buf, struct waitqueue *wq, int starttime)
{
int	x,y,z;
int	secs;
struct	waitqueue *temp;
char	s[80];

temp = wq;
x = 1;
z = starttime;
while(temp != NULL)
	{
	secs = time(NULL) - temp->lastupdate;
	sprintf(s,"(%-3d)  %-16.16s  %-3d   %-4d\n",x,temp->host,secs,z);
	strcat(buf,s);
	if(temp->lastupdate > time(NULL) - 60)
		z = z + 5*60;
	temp = temp->next;
	x++;
	}
}


/* waitqueuestat
This function takes a waitqueue pointer and a host, computes how long that
host will probably have to wait and returns a string formatted:
timeleft, number of users ahead in the queue,

*/
waitqueuestat(char *buf, struct waitqueue *wq, int timeleft, char *host)
{
int	x,y,z;
struct	waitqueue *temp;

z = timeleft;
x = 0;
temp = wq;
while(temp != NULL)
	{
	if(strcmp(temp->host,host) == 0)
		{
		sprintf(buf,"%d	%d",z,x);
		temp->lastupdate = time(NULL);
		return(0);
		}
	x++;	
	if(temp->lastupdate > time(NULL) - 60)
		z = z + 5*60;
	temp = temp->next;
	}
return(1);
}

int	rccarlock(struct rccardata *rccar, char *host)
{
rccarwaitenqueue(host,rccar);
	
return(1);
}

getslavedata(char *buf)
{
int	x,y,z;
long	ti;
char	s[2000],t[2000];
struct	sdlink	*temp;

time(&ti);
strcpy(s,"");
for(z=0;z<100;z++)
	{
	if(slavedata[z].status)
		{
		sprintf(t,"%d: %s ",z,slavedata[z].host);
		
		temp = switches;
		while(temp != NULL)
			{
			if(temp->sd.relaythread == z && temp->sd.control == 3)
				{
				strcat(t,temp->sd.name);
				strcat(t," ");
				}
			temp=temp->next;
			}
		strcat(s,t);
		strcat(s,"(");
		if(slavedata[z].pongwait)
			strcat(s,"*");
		sprintf(t,"%d)\n",ti-slavedata[z].lastping);
		strcat(s,t);
		}
	}
strcpy(buf,s);

}

getlampdata(char *buf,int mode)
{
int	x,y,z;
char	s[2000],t[20000];
char	w[200];
struct	sdlink	*temp;

temp=switches;
strcpy(t,"");
for(y=0;y<17;y++)
	for(x=0;x<17;x++)
		{
		if(X10mods[y][x].label[0] != 0)
			{
			sprintf(w,"%c%d",y+'A',x);	
			if(mode)
				{
				sprintf(s,"%d: %s;%s;%s;%d\n",X10mods[y][x].available,w,X10mods[y][x].label,X10mods[y][x].camname[0],X10mods[y][x].icon);
				strcat(t,s);
				}
			else if(X10mods[y][x].available)
				{
				sprintf(s,"%s;%s;%d\n",w,X10mods[y][x].label,X10mods[y][x].icon);
				strcat(t,s);
				}
			}
		}
while(temp != NULL)
	{
	if(mode)
		{
		sprintf(s,"%s;%s;%s;%d\n",temp->sd.name,temp->sd.label,temp->sd.camname[0],temp->sd.icon);
		strcat(t,s);
		}
	else if(temp->sd.available)
		{
		sprintf(s,"%s;%s;%s;%d\n",temp->sd.name,temp->sd.label,temp->sd.camname[0],temp->sd.icon);
		strcat(t,s);
		}
	temp = temp->next;
	}
strcpy(buf,t);
}

getmotordata(char *buf,int mode)
{
int	x,y,z;
char	s[2000],t[20000];
char	w[200];
struct	mdlink	*temp;

temp=motors;
strcpy(t,"");
while(temp != NULL)
	{
	if(temp->md.available)
		{
		sprintf(s,"%s;%s;%d\n",temp->md.name,temp->md.label,0);
		strcat(t,s);
		}
	temp = temp->next;
	}
strcpy(buf,t);
}

slave_configure_lamps(int	mode)
{
int	x,y,z;
char	s[2000],t[20000];
char	w[200];
struct	sdlink	*temp;
struct	mdlink  *temp2;

temp=switches;
temp2 = motors;
strcpy(t,"");
if(mode == 1)
	{
	sprintf(s,"SLAVE\n");
	write(myslave.msgsock,s,strlen(s)+1);
	return(1);
	}
//sleep(10);
while(temp2 != NULL)
	{
	if(temp2->md.available && temp2->md.control == 1)
		{
		sprintf(s,"SL%s%s ADDMOTOR \"%s\" %s\n",clientname,temp2->md.name,temp2->md.label,temp2->md.name);
		printf("Sending s=.%s.\n",s);
		write(myslave.msgsock,s,strlen(s)+1);
//		sleep(1);
		
		}
	temp2 = temp2->next;
	}
while(temp != NULL)
	{
	if(temp->sd.available && temp->sd.control == 1)
		{
		char	s243[160];
		http_encode(s243,temp->sd.camname[0]);
		sprintf(s,"SL%s%s ADDSWITCH \"%s\" %s switchtype=%d&icon=%d&startstatus=%d&times0=%d&times1=%d&times2=%d&times3=%d&times4=%d&camname=%s\n",clientname,temp->sd.name,temp->sd.label,temp->sd.name,temp->sd.switchtype,temp->sd.icon,temp->sd.startstatus,temp->sd.times[0],temp->sd.times[1],temp->sd.times[2],temp->sd.times[3],temp->sd.times[4],s243);
		printf("Sending s=.%s.\n",s);
		write(myslave.msgsock,s,strlen(s)+1);
//		sleep(3);
		}
	temp = temp->next;
	}
for(y=0;y<17;y++)
	for(x=0;x<17;x++)
		{
		if(X10mods[y][x].label[0] != 0)
			{
			sprintf(w,"%c%d",y+'A',x);	
			if(X10mods[y][x].available)
				{
				sprintf(s,"SL%s%s ADDSWITCH \"%s\" %s icon=%d&camname=%s\n",clientname,w,X10mods[y][x].label,w,X10mods[y][x].icon,X10mods[y][x].camname[0]);
				printf("Sending s=.%s.\n",s);
				write(myslave.msgsock,s,strlen(s)+1);
//				sleep(3);
				}
			}
		}
sleep(2);
sprintf(s,"STATUS LISTLAMPS 0 spaz\n");
write(myslave.msgsock,s,strlen(s)+1);
}

master_configure_lamps(int msgsock, int slavenum, int boredvalue)
{
int	x,y,z;
char	s[2000],t[20000];
char	w[200];
struct	sdlink	*temp;
struct	mdlink  *temp2;

temp=switches;
temp2 = motors;
strcpy(t,"");
printf("Socket in master function: %d\n",msgsock);
sleep(5);
while(temp2 != NULL)
	{
	if(temp2->md.available && !(temp2->md.control == 3 && temp2->md.relaythread == slavenum))
		{
		sprintf(s,"MASTER%s ADDMOTOR \"%s\" %s\n",temp2->md.name,temp2->md.label,temp2->md.name);
		printf("Sending s=.%s.\n",s);
		write(msgsock,s,strlen(s)+1);
//		sleep(3);
		
		}
	temp2 = temp2->next;
	}
while(temp != NULL)
	{
	if(boredvalue)
		{
		if(temp->sd.bored)
			z = 1;
		else
			z = 0;
		}	
	else
		z = 1;
	if(temp->sd.available && !(temp->sd.control == 3 && temp->sd.relaythread == slavenum) && z)
		{
		sprintf(s,"MASTER%s ADDSWITCH \"%s\" %s icon=%d&startstatus=%d&times0=%d&times1=%d&times2=%d&times3=%d&times4=%d\n",temp->sd.name,temp->sd.label,temp->sd.name,temp->sd.icon,temp->sd.startstatus,temp->sd.times[0],temp->sd.times[1],temp->sd.times[2],temp->sd.times[3],temp->sd.times[4]);
		printf("Sending s=.%s.\n",s);
		write(msgsock,s,strlen(s)+1);
//		sleep(3);
		}
	temp = temp->next;
	}
for(y=0;y<17;y++)
	for(x=0;x<17;x++)
		{
		if(boredvalue)
			{
			if(X10mods[y][x].bored)
				z = 1;
			else
				z = 0;
			}	
		else
			z = 1;
		if(X10mods[y][x].label[0] != 0)
			{
			sprintf(w,"%c%d",y+'A',x);	
			if(X10mods[y][x].available && z)
				{
				sprintf(s,"MASTER%s ADDSWITCH \"%s\" %s icon=%d\n",w,X10mods[y][x].label,w,X10mods[y][x].icon);
				printf("Sending s=.%s.\n",s);
				write(msgsock,s,strlen(s)+1);
//				sleep(3);
				}
			}
		}
}


int	thread_start(int *arg)
{
int	x,y,z;
int	msgsock;

msgsock = *arg;

}


int	misc_commands(char *buf, int *insanity,int *shutdownmarker,int inslamp,char *s)
{
if(strcmp(buf,"ADDBIGINS") == 0)
	{
	printf("Adding insanity.\n");
	*insanity = *insanity + inslamp*10;
	}
if(strcmp(buf,"ADDINS") == 0)
	{
	printf("Adding insanity.\n");
	*insanity = *insanity + inslamp*3;
	}
if(strcmp(buf,"INSRESET") == 0)
	{
	printf("Restting insanity level.\n");
	*insanity=0;
	}

/* No longer used
		if(strcmp(buf,"SPRESET") == 0)
			{
			printf("Restting sprinkler timer.\n");
			sprinksd.timeused = 0;
			}
		if(strcmp(buf,"SPCLEAR") == 0)
			{
			printf("Clearing sprinkler timer.\n");
			sprinksd.timeused=2000;
			}
*/

if(strcmp(buf,"SHUTDOWN") == 0)
	{
	*shutdownmarker = 1;
	}
if(strcmp(buf,"GETLAMPDATA") == 0)
	{
	getlampdata(s,0);
	}
if(strcmp(buf,"GETMOTORDATA") == 0)
	{
	getmotordata(s,0);
	}
if(strcmp(buf,"GETLAMPDATA2") == 0)
	{
	getlampdata(s,1);
	}
if(strcmp(buf,"FULLSTATUS") == 0)
	{
	char	t1[2000];
	sprintf(t1,"Lamps:\n");
	getlampdata(s,1);
	strcat(t1,s);
	strcat(t1,"Motors:\n");
	getmotordata(s,1);
	strcat(t1,s);
	strcat(t1,"Slaves:\n");
	getslavedata(s);
	strcat(t1,s);
	strcpy(s,t1);
	
	}
}

int	message_commands(char *command, char *message, char *host, char *s)
{
printf("Message section.  command = .%s. host = .%s. message= .%s.\n",command,host,message);
if(strcmp(command,"SEND") == 0)
	{
	messageenqueue(message,host);
	}
if(strcmp(command,"GET") == 0)
	{
	messagedequeue(s,host);
	}
}

rccar_commands(char *name,char *command,char *host,struct rccardata *rccar,char *s,int *compstate,int insanity, int *state2)
{
int strike;
if(strcmp(command,"NUDGEF") == 0)
	{
	strike = rccarenqueue(0,0,1,host, rccar);
	
	}
if(strcmp(command,"PUSHF") == 0)
	{
	strike = rccarenqueue(0,1,1,host, rccar);
	}
if(strcmp(command,"NUDGEB") == 0)
	{
	strike = rccarenqueue(0,0,0,host, rccar);
	}
if(strcmp(command,"PUSHB") == 0)
	{
	strike = rccarenqueue(0,1,0,host, rccar);
	}
if(strcmp(command,"TURNLEFT") == 0)
	{
	strike = rccarenqueue(1,0,1,host,rccar);
	/* Turn Left */
	}
if(strcmp(command,"TURNRIGHT") == 0)
	{
	strike = rccarenqueue(1,0,2,host,rccar);
	/* Turn Right */
	}
if(strcmp(command,"CENTER") == 0)
	{
	/* Center wheels */
	strike = rccarenqueue(1,0,0,host,rccar);
	}
if(strcmp(command,"UPLEFT") == 0)
	strike = rccarenqueue(2,0,5,host, rccar);
if(strcmp(command,"UPRIGHT") == 0)
	strike = rccarenqueue(2,0,6,host, rccar);
if(strcmp(command,"DOWNLEFT") == 0)
	strike = rccarenqueue(2,0,1,host, rccar);
if(strcmp(command,"DOWNRIGHT") == 0)
	strike = rccarenqueue(2,0,2,host, rccar);
if(strcmp(command,"LOCK") == 0)
	{
	printf("Locking the RC car for %s\n",host);
	strike = rccarwaitenqueue(host,rccar);
	}	
if(strcmp(command,"WAITQUEUE") == 0)
	{
	printf("Getting wait queue for %s\n",host);
	strike = waitqueuestat(s,rccar->wq,rccar->lockexp - time(NULL), host);	
	printf("Got it: .%s.\n",s);
	if(strike)
		{
		if(rccar->lock && strcmp(rccar->lockhost,host) == 0)
			sprintf(s,"%d -1",rccar->lockexp - time(NULL));
		else
			sprintf(s,"-1 -1");
		}
	}
else if(strcmp(command,"GETWQLIST") == 0)
	{
	if(rccar->lock)
		{
		sprintf(s,"(Num)  Host             Last  Left\n(%-3d)  %-16.16s  %-3d   %-4d\n",0,rccar->lockhost,time(NULL) - rccar->heartbeat,rccar->lockexp - time(NULL));
		waitqueuelist(s,rccar->wq,rccar->lockexp - time(NULL));
		}
	else
		{
		sprintf(s,"\n");
		}
	}
else
	{
	if(strike == 0)
		strike=messagestrike(host);
	printf("Getting state.  Strike = %d\n",strike);
	getSTATE(s, insanity, compstate,strike);
	}
}

checkswitches(char *name,char *command,int *insanity,char *host,int *compstate,char *s,int inslamp)
{
struct	switchdata *sd;
sd = searchswitch(name);
if(sd != NULL)
	{
	if(strcmp(command,"ON") == 0)
		lampon(sd,getstatevar(sd->switchbase),insanity,inslamp);
	if(strcmp(command,"OFF") == 0)
		lampoff(sd,getstatevar(sd->switchbase));
	if(strcmp(command,"TOGGLE") == 0)
		{
		if(sd->status)
			lampoff(sd,getstatevar(sd->switchbase));
		else
			lampon(sd,getstatevar(sd->switchbase),insanity,inslamp);
		}
	if(strcmp(command,"AVAIL") == 0)
		sd->available=1;
	if(strcmp(command,"UNAVAIL") == 0)
		sd->available=0;
	getSTATE(s, *insanity, compstate,messagestrike(host));
	}
}

checkrccars(char *name,char *command,int *insanity,char *host,int *compstate,char *s,int inslamp)
{
int strike;
struct	rccardata *rccar;
rccar = searchrccar(name);
if(rccar != NULL)
	{
	printf("Found an rc car.\n");


	if(strcmp(command,"NUDGEF") == 0)
		{
		strike = rccarenqueue(0,0,1,host, rccar);
		
		}
	if(strcmp(command,"PUSHF") == 0)
		{
		strike = rccarenqueue(0,1,1,host, rccar);
		}
	if(strcmp(command,"NUDGEB") == 0)
		{
		strike = rccarenqueue(0,0,0,host, rccar);
		}
	if(strcmp(command,"PUSHB") == 0)
		{
		strike = rccarenqueue(0,1,0,host, rccar);
		}
	if(strcmp(command,"TURNLEFT") == 0)
		{
		strike = rccarenqueue(1,0,1,host,rccar);
		/* Turn Left */
		}
	if(strcmp(command,"TURNRIGHT") == 0)
		{
		strike = rccarenqueue(1,0,2,host,rccar);
		/* Turn Right */
		}
	if(strcmp(command,"CENTER") == 0)
		{
		/* Center wheels */
		strike = rccarenqueue(1,0,0,host,rccar);
		}
	if(strcmp(command,"UPLEFT") == 0)
		strike = rccarenqueue(2,0,5,host, rccar);
	if(strcmp(command,"UPRIGHT") == 0)
		strike = rccarenqueue(2,0,6,host, rccar);
	if(strcmp(command,"DOWNLEFT") == 0)
		strike = rccarenqueue(2,0,1,host, rccar);
	if(strcmp(command,"DOWNRIGHT") == 0)
		strike = rccarenqueue(2,0,2,host, rccar);
	if(strcmp(command,"LOCK") == 0)
		{
		printf("Locking the RC car for %s\n",host);
		strike = rccarwaitenqueue(host,rccar);
		}	
	if(strcmp(command,"WAITQUEUE") == 0)
		{
		printf("Getting wait queue for %s\n",host);
		strike = waitqueuestat(s,rccar->wq,rccar->lockexp - time(NULL), host);	
		printf("Got it: .%s.\n",s);
		if(strike)
			{
			if(rccar->lock && strcmp(rccar->lockhost,host) == 0)
				sprintf(s,"%d -1",rccar->lockexp - time(NULL));
			else
				sprintf(s,"-1 -1");
			}
		}
	else if(strcmp(command,"GETWQLIST") == 0)
		{
		if(rccar->lock)
			{
			sprintf(s,"(Num)  Host             Last  Left\n(%-3d)  %-16.16s  %-3d   %-4d\n",0,rccar->lockhost,time(NULL) - rccar->heartbeat,rccar->lockexp - time(NULL));
			waitqueuelist(s,rccar->wq,rccar->lockexp - time(NULL));
			}
		else
			{
			sprintf(s,"\n");
			}
		}
	else
		{
		if(strike == 0)
			strike=messagestrike(host);
		printf("Getting state.  Strike = %d\n",strike);
		getSTATE(s, *insanity, compstate,strike);
		}
	}
}

checkmotors(char *name,char *command,int *insanity,char *host,int *compstate,char *value,char *s)
{
struct	motordata *md;
md = searchmotor(name);
if(md != NULL)
	{
	int	f1;
	f1 = atoi(value);
	if(strcmp(command,"ZOOMIN") == 0)
		zoom(0,md,getstatevar(md->motorbase),f1);
	if(strcmp(command,"ZOOMOUT") == 0)
		zoom(1,md,getstatevar(md->motorbase),f1);
	if(strcmp(command,"PANLEFT") == 0)
		pan(1,md,getstatevar(md->motorbase),1);
	if(strcmp(command,"PANRIGHT") == 0)
		pan(0,md,getstatevar(md->motorbase),1);
	if(strcmp(command,"ZOOMCALI") == 0)
		zoomcalibrate(md);
	if(strcmp(command,"ZOOMTO") == 0)
		zoomto(f1,md,getstatevar(md->motorbase));

	if(strcmp(command,"AVAIL") == 0)
		md->available=1;
	if(strcmp(command,"UNAVAIL") == 0)
		md->available=0;
	getSTATE(s, *insanity, compstate,messagestrike(host));
	}
}

checkx10(char *name,char *command,int *insanity,char *host,int *compstate,char *value,char *s)
{
int	x,y,z;
if(searchx10(name,&y,&x))
	{
	if(strcmp(command,"ON") == 0)
		{
		sprintf(s,"X10ON %c%2.2d %d %s",y+'A',x,atoi(value),host);
		x10enqueue(s,host);
		}
	if(strcmp(command,"OFF") == 0)
		{
		sprintf(s,"X10OF %c%2.2d %d %s",y+'A',x,atoi(value),host);
		x10enqueue(s,host);
		}
	if(strcmp(command,"TOGGLE") == 0)
		{
		if(X10mods[y][x].state)
			sprintf(s,"X10OF %c%2.2d %d %s",y+'A',x,atoi(value),host);
		else
			sprintf(s,"X10ON %c%2.2d %d %s",y+'A',x,atoi(value),host);
		x10enqueue(s,host);
		}
	if(strcmp(command,"AVAIL") == 0)
		X10mods[y][x].available=1;
	if(strcmp(command,"UNAVAIL") == 0)
		X10mods[y][x].available=0;
	getSTATE(s, *insanity, compstate,messagestrike(host));
	}
}


int	slavecommands(char *buf, struct slaveinfo *sld,int slavenum, char *name, char *command, char *value, char *host, char *message)
{
char 	s[2000];
struct	switchdata	tmpsd;
struct	motordata	tmpmd;
struct	rccardata	tmprc;

printf("Socket: %d  xxx = %d\n",sld->msgsock,slavenum);
if(strncmp(buf,"PING",4) == 0)
	{
	// slave pinged us.  Send a pong.
	sprintf(s,"PONG\n");
	write(sld->msgsock,s,strlen(s)+1); 	
	sld->pongwait = 0;
	printf("Got a ping, sending pong.\n");
	}
if(strncmp(buf,"PONG",4) == 0)
	{
	sld->pongwait = 0;
	printf("Got a pong.\n");
	}
if(strcmp(name,"STATUS") == 0)
	{
	if(strcmp(command,"LISTLAMPS") == 0)
		{
		int	xxx;
		printf("GOT HERE!!!!!!!!\n");
		xxx = atoi(value);
		sprintf(s,"PING\n");
		write(sld->msgsock,s,strlen(s)+1);
		printf("Socket: %d\n",sld->msgsock);
		sleep(1);
		master_configure_lamps(sld->msgsock,slavenum,xxx);
		}
	}
if(strcmp(command,"ADDSWITCH") == 0)
	{
	printf("Got ADDSWITCH.\n");
	sld->pongwait = 0;
	initswitch(&tmpsd);
	strcpy(tmpsd.name,name);
	strcpy(tmpsd.relayname,host);
	strcpy(tmpsd.label,value);
	tmpsd.control=3;
	tmpsd.relaythread=slavenum;
	findtoken("icon",message,s);
	tmpsd.icon = atoi(s);
	findtoken("switchtype",message,s);
	tmpsd.switchtype = atoi(s);
	findtoken("camname",message,s);
	filter(s,tmpsd.camname[0]);
	addswitch(tmpsd);
	}	
if(strcmp(command,"ADDRCCAR") == 0)
	{
	printf("Got ADDSWITCH.\n");
	sld->pongwait = 0;
	rccarinit(&tmprc);
	strcpy(tmprc.name,name);
	strcpy(tmprc.relayname,host);
	strcpy(tmprc.label,value);
	tmprc.control=3;
	tmprc.relaythread=slavenum;
	findtoken("type",message,s);
	tmprc.type = atoi(s);
	addrccar(tmprc);
	}	
if(strcmp(command,"ADDMOTOR") == 0)
	{
	printf("Got ADDMOTOR.\n");
	sld->pongwait = 0;
	motorinit(&tmpmd);
	strcpy(tmpmd.name,name);
	strcpy(tmpmd.relayname,host);
	strcpy(tmpmd.label,value);
	tmpmd.control=3;
	tmpmd.relaythread=slavenum;
	addmotor(tmpmd);
	}	
if(strncmp(buf,"SLAVECLOSE",10) == 0)
	{
	numslaves--;
	close(sld->msgsock);
	// remove all of slaves's lamps.
	purge_switches(slavenum);
	sld->status=0;
	}
}


doslavestuff(int *insanity, struct slaveinfo *sld, int slavenum, char *s, int *compstate,int inslamp)
{
int	rval;
char	buf[20000];
char	name[200];
char	command[200];
char	host[200];
char	value[200];
char	message[20000];
do
	{
	rval = newgetcommand(commandsbuf, buf,name,command,value,host,message);
	if(rval <= 0)
		break;
	printf("Got command. buf=.%s.\n",buf);

	slavecommands(buf,sld,slavenum,name,command,value,host,message);
	checkswitches(name,command,&insanity,host,compstate,s,inslamp);
	checkrccars(name,command,&insanity,host,compstate,s,inslamp);
	checkmotors(name,command,&insanity,host,compstate,value,s);
	checkx10(name,command,&insanity,host,compstate,value,s);
	} while (rval > 0);
}

int	normalcommands(int msgsock,int *insanity, int *shutdownmarker,int *compstate, int inslamp,char *clienthost)
{
int	x,y,z;
int	rval;

char	s[10000];
char 	buf[2000];
char	name[20], command[20], value[40], host[20];
char	message[3000];


struct	rccardata rccar;	// data for the rccar
struct	switchdata *sd;
struct	motordata *md;
struct	X10data	*x10d;


strcpy(s,"");
rval = getcommand(msgsock, buf,name,command,value,host,message);
printf("buf=.%s.\n",buf);
printf("Command: %s %s %s %s .%s.\n",name, command,value,host,message);
if(rval == -1)
	return(-1);	
misc_commands(buf,insanity,shutdownmarker,inslamp,s);

/*
The next commands are for backwards compatibility until all features and
cgi scripts are updated.
*/

/*
if(strcmp(buf,"CALI") == 0)
	{
	printf("Beginning calibration.\n");
	for(y=0;y<70;y++) 
		{
		pan(0,&officerotor,&state,1);
		usleep(100000);
		}	
	officerotor.currentstep=0;
	for(y=0;y<25;y++) 
		{
		pan(1,&officerotor,&state,1);
		usleep(100000);
		}
	}
*/

/* Get the last motion sensor tripped */
/*
if(strcmp(buf,"LASTMS") == 0)
	{
	s[0] = lastms + '0';
	s[1] = 0;
	}
*/
if(strcmp(name,"MESSAGE") == 0)
	message_commands(command, message,host,s);

/* This is part of the new routines, just add it later.
*/

/*
if(strcmp(name,"RCCAR") == 0)
	{
	rccar_commands(name,command,host,&rccar,s,compstate,insanity, state2);
	}
*/
		
			
/*
Now for the new routines
*/
checkswitches(name,command,insanity,host,compstate,s,inslamp);

checkrccars(name,command,&insanity,host,compstate,s,inslamp);

checkmotors(name,command,insanity,host,compstate,value,s);

checkx10(name,command,insanity,host,compstate,value,s);

if(strncmp(buf,"SILENTSTATE",11) == 0)
	{
	getSTATE(s, *insanity, compstate,2);
	}
/*
if(strcmp(buf,"TIMERESET") == 0)
	{
	resettimes(&officesd,&sprinksd );
	}
*/

/*
slaving function.
Client lightserver program will slave itself to this one using this command.
This socket connection will be maintained.
A new thread will open.
the main thread will continue as normal
*/
if(strcmp(buf,"SLAVE\n") == 0)
	{
	int	xxx;
	numslaves++;
	for(xxx=0;xxx<100 && slavedata[xxx].status;xxx++);
	slavedata[xxx].msgsock = msgsock;
	slavedata[xxx].status = 1;
	slavedata[xxx].getdevstat = 0;
	slavedata[xxx].postdevstat = 0;
	strncpy(slavedata[xxx].host,clienthost,39);
	slavedata[xxx].host[39] = 0;
	printf("About to configure master devices.\n");
	//master_configure_lamps(slavedata[xxx].msgsock);
	printf("done configuring master devices.\n");
	return(0);
	}


if(strcmp(name,"STATUS") == 0)
	{
	int	camnum;
	if(strcmp(command,"STATE") == 0)
		{
		camnum = atoi(value);
		newgetSTATE(s, *insanity, compstate,messagestrike(host),camnum,host);
		}
	if(strcmp(command,"CAMCOUNT") == 0)
		{
		int	camnum;
		camnum = atoi(value);
		lsgetcamcount(camnum,s);
		printf("Got back.\n");
		}
	if(strcmp(command,"LISTCAMHOSTS") == 0)
		{
		int camnum;
		camnum = atoi(value);
		getlistcamhosts(camnum,s);
		}
	}
if(strcmp(name,"STATE") == 0)
	{
	getSTATE(s, *insanity, compstate,messagestrike(host));
	}
printf("About to write.\n");
if(write(msgsock, s, strlen(s)+1) == -1)
	perror("Writting on stream socket ");
printf("Done writing.  about to close.\n");
close(msgsock);
return(1);
}




main(argc, argv)
int	argc;
char 	*argv[];
{
int	sock;
int	length;
struct	sockaddr_in server;
struct	sockaddr_in client;
int	msgsock;
int	slavedelay;
char	buf[1024];
char	s[20000],t[1024];
char	clienthost[80];
int	x,y,z;
int	rval;
int	i;
time_t	lt;
struct	tm	*tim;
struct	tm 	*lasttim;
FILE	*output;
int	size;
int	lasthour;

char	name[20], command[20], value[40], host[20];

int	lastcontact;	/* stores the last contact tapped */
int	currentstep;	/* What is the current step; for bounding */	

struct	motordata officerotor;	// data for the office cam rotor
struct	rccardata rccar;	// data for the rccar
struct	switchdata officesd;
struct	switchdata *sd;
struct	motordata *md;
struct	switchdata sprinksd;
struct	X10data	*x10d;

long	counter;	/* Lamp time counter */
long	wcounter;	/* Wheel time counter */
long	spcounter;

int	status; 	/* Lamp status */
int	wstatus;	/* Wheel status */
int	spstatus;

int	lastms;		/* motion sensor tracking */

long	sprinkseconds;	/* number of seconds sprinkler has run today */
long 	sprinklermax;	/* max # of seconds to run the sprinkler each day */
long	starttime;	/* time server was started at, for tracking sprinkler */	
long	lasttime;	/* last time checked */
long	currenttime;	/* time right now */
long	timetmp;

int	insanity;	/* the insanity value */

int	inslamp;	/* insanity increase when a lamp is turned on. */
int	inssub;		/* insanity value to decrease each 10 seconds */

long	insanitytime;

int	shutdownmarker;
char	message[3000];




int	compstate[20];
long	compchecktime;

fd_set	rfds;
struct	timeval	tv;
int	retval;

unsigned	char	*address_holder;



signal(SIGCHLD,child_handler);
signal(SIGPIPE,pipecleaner);

strcpy(commandsbuf,"");

numslaves=0;
slavedelay=0;
slaved=0;
socketlink=NULL;

lasthour = 0;

insanity = 0;
inslamp = 5;
inssub = 10;

lastms = 1;

bans = NULL;
mqlist = NULL;

shutdownmarker = 0;

starttime = time(NULL);
insanitytime=starttime;

sprinklermax = 30*60;	/* 15 minutes per day max */
sprinkseconds = sprinklermax;

compchecktime=0;
myslave.pongwait=0;
myslave.lastping=starttime;

for(z=0;z<200;z++)
	{
	banwatchlist[z].host[0] = 0;
	banwatchlist[z].count = 0;
	banwatchlist[z].warned = 0;
	rccarwatchlist[z].host[0] = 0;
	rccarwatchlist[z].count = 0;
	rccarwatchlist[z].warned = 0;
	
	}
for(z=0;z<100;z++)
	{
	slavedata[z].status=0;
	slavedata[z].pongwait = 0;
	slavedata[z].lastping=starttime;
	}
for(z=0;z<20;z++)
	compstate[z] = 0;

for(z=0;z<17;z++)
	for(y=0;y<17;y++)
		{
		X10mods[z][y].state = 0;
		X10mods[z][y].limit = 0;
		X10mods[z][y].control = 1;
		}

printf("about to read.\n");
readconfig();
printf("Done reading.\n");
printconfig();

status=0;
wstatus=0;
spstatus=0;
state=0;
state2=0;
currentstep=25;
lastcontact=-1;

initswitch(&officesd);

officesd.port = 0;
initswitch(&sprinksd);
sprinksd.port=5;
sprinksd.startstatus = 0;
sprinksd.times[0] = 6*60;
sprinksd.times[1] = 19*60;
sprinksd.maxsecs = 30*60;

officerotor.currentstep=25;
officerotor.lastcontact = -1;
officerotor.sequence[0] = 4;
officerotor.sequence[1] = 8;
officerotor.sequence[2] = 2;
officerotor.sequence[3] = 16;
officerotor.numsequence = 4;
officerotor.motorbase = base;
officerotor.zoominpin = 2;
officerotor.zoomoutpin = 4;

rccar.commonport=16;
rccar.forwardport=2;
rccar.backwardport=1;
rccar.leftport = 8;
rccar.rightport = 4;
rccar.carbase = base2;
rccar.nudgef = SLOW;
rccar.nudgeb = SLOW;
rccar.pushf = MEDIUM;
rccar.pushb = FAST;
rccar.wstatus = 0;
rccar.wcounter = 0;
rccar.eq = NULL;
rccar.wq = NULL;
rccar.lock = 0;
rccar.lockexp = 0;

testswitches();

if(slaved)
	{
	myslave.msgsock = connect_socket(myslave.host,5201);
	if(myslave.msgsock == -1)
		{
		printf("Unable to connect as a slave.  Exiting.\n");
		exit(1);
		}
	printf("Connected as slave.\n");
	slavedelay=time(NULL) + 10;
	myslave.getdevstat=0;
	myslave.postdevstat=0;
	slave_configure_lamps(1);
	}





sock = sockinit(listenport);
do
	{
	timetmp = time(NULL);
	tim = localtime(&timetmp);	

/*
This part will be redone.  A linked list of open sockets will be maintained.
getconnection will set up a socket.  a select will then determine if anything
is ready.  If not, it will check other sockets.  Only once something is found
will it be processed.

Processing will take the command and push it on a queue.  Command queues will
consist of all the variables and the slave/master the command is for.

First, break out the main loop into smaller functions.  60 lines max.
*/
	do
		{
		getconnection(sock,&socketlink);
		msgsock = scansockets(&socketlink,clienthost);
//		printf("scansockets done.  msgsock= %d\n",msgsock);
		if(msgsock > 0)
			{
			x = normalcommands(msgsock,&insanity, &shutdownmarker, compstate, inslamp,clienthost);
			if(x == 0)
				deletesocket(msgsock,0,&socketlink);
			else
				deletesocket(msgsock,1,&socketlink);
			}
		}
	while(msgsock > -1);
/*
conditions of while loop:
loop for at least 200 ms (unless nothing is in the queue) and no longer than 1
second.  Otherwise, break out of the loop when nothing is active.
*/

/* This next part is temporary.  For now, it will dequeue one entry from the
rccar queue and execute it, then return to the loop.  This will eventually
be handled by a separate thread.
*/
	if(shutdownmarker)
		{
		if(slaved)
			{
			sprintf(s,"SLAVECLOSE\n");
			write(myslave.msgsock,s,strlen(s)+1);
			close(myslave.msgsock);
			}
		close(sock);
		exit(0);
		}
	rccardequeue(&rccar);
// x10 devices should be dequeued ONLY when there is no other pending activity.
	x10dequeue(&insanity, inslamp);

/*
Check all slaves to see if there are any incoming messages.  This will be
rare, just to add new appliances at the beginning.
*/
	if(numslaves > 0)
		{
		int	xxx;
		long	tempt;
		int	rval;
		fd_set	rfds;
		struct	timeval	tv;
		int	retval;
		struct	switchdata	tmpsd;
		struct	motordata	tmpmd;
		for(xxx=0;xxx<100;xxx++)
			{
			if(!slavedata[xxx].status)
				continue;
			FD_ZERO(&rfds);
			FD_SET(slavedata[xxx].msgsock,&rfds);
			/* Wait 1 second for data, then proceed with other stuff */
		
			tv.tv_sec=0;
			tv.tv_usec=1;	
			retval = select(slavedata[xxx].msgsock + 1, &rfds, NULL, NULL, &tv);
			if(retval)		/* we got data */
				{
				rval = readcommand(slavedata[xxx].msgsock, commandsbuf);
				printf("xxx= %d\n",xxx);
				if(rval > 0) 
					{
					printf("I'm here.\n");
					doslavestuff(&insanity,&slavedata[xxx],xxx,s,compstate,inslamp);
/*
					do
						{	
						rval = newgetcommand(commandsbuf, buf,name,command,value,host,message);
						if(rval <= 0)
							break;
						printf("Got command. buf=.%s. name=%s command=.%s. host=.%s. xxx=%d  socket=%d\n",buf,name,command,host,xxx,slavedata[xxx].socket);
					
						checkswitches(name,command,&insanity,host,compstate,s,inslamp);
						checkmotors(name,command,&insanity,host,compstate,value,s);
						checkx10(name,command,&insanity,host,compstate,value,s);
						slavecommands(buf,&slavedata[xxx],xxx,name,command,value,host,message);
						} while (rval > 0);
*/
					}
				else if(rval == -1 || rval == 0)
					{
					printf("Doing the numslaves thing.\n");
					numslaves--;
					close(slavedata[xxx].msgsock);
					// remove all of slaves's lamps.
					purge_switches(xxx);
					slavedata[xxx].status=0;
					}
				}
			time(&tempt);
			if(slavedata[xxx].pongwait && slavedata[xxx].lastping + 60 < tempt)
				{
				printf("Normally I'd kill it here.\n");
// Reimplement this part if it breaks something
				printf("I'm killing it!!!");
				sleep(1);
				// no response from slave.  Kill it off
				numslaves--;
				close(slavedata[xxx].msgsock);
				// remove all of slaves's lamps.
				purge_switches(xxx);
				slavedata[xxx].status=0;
				slavedata[xxx].pongwait=0;
				}
			if(slavedata[xxx].lastping + 120 < tempt)
				{
				// send a ping every 5 minutes.
				sprintf(s,"PING\n");
				write(slavedata[xxx].msgsock,s,strlen(s)+1); 	
				slavedata[xxx].lastping = tempt;
				slavedata[xxx].pongwait = 1;
				printf("PINGING! xxx=%d\n",xxx);
				}
			}
		
		}

	if(slaved)
		{
		long	tempt;
		int	rval;
		fd_set	rfds;
		struct	timeval	tv;
		int	retval;
		struct	switchdata	tmpsd;
		struct	motordata	tmpmd;
		
		FD_ZERO(&rfds);
		FD_SET(myslave.msgsock,&rfds);
		/* Wait 1 second for data, then proceed with other stuff */
	
		tv.tv_sec=0;
		tv.tv_usec=1;	
		retval = select(myslave.msgsock + 1, &rfds, NULL, NULL, &tv);
		if(retval)		/* we got data */
			{
			rval = readcommand(myslave.msgsock, commandsbuf);
			if(rval > 0 )
				{
				doslavestuff(&insanity,&myslave,-1,s,compstate,inslamp);
/*
				do
					{
					rval = newgetcommand(commandsbuf, buf,name,command,value,host,message);
					if(rval <= 0)
						break;
					printf("Got command. buf=.%s.\n",buf);

					slavecommands(buf,&myslave,-1,name,command,value,host,message);
					checkswitches(name,command,&insanity,host,compstate,s,inslamp);
					checkmotors(name,command,&insanity,host,compstate,value,s);
					checkx10(name,command,&insanity,host,compstate,value,s);
					} while (rval > 0);
*/
				}
			else
				{
				printf("oh dear god no.  I lost my master.  Reconnecting.\n");
				purge_switches(-1);
				myslave.msgsock = connect_socket(myslave.host,5201);
				while(myslave.msgsock == -1)
					{
					myslave.msgsock = connect_socket(myslave.host,5201);
					}
				printf("Reconnected as slave.\n");
				slavedelay = time(NULL) + 30;
				slave_configure_lamps(1);
				
				
				}
			}
		time(&tempt);
		if(myslave.pongwait && myslave.lastping + 60 < tempt)
			{
			printf("I never got a pong.  Closing down.\n");
// reimplement this if it breaks
			// no response from master.  FLEE!!!! 
			close(myslave.msgsock);
			myslave.status=0;
			myslave.pongwait = 0;
			// should get all excited on the next loop and reconnect.
			}
		if(myslave.lastping + 120 < tempt)
			{
			// send a ping every 5 minutes.
			sprintf(s,"PING\n");
			printf("SENDING A PING!!!!!\n");
			write(myslave.msgsock,s,strlen(s)+1); 	
			myslave.lastping = tempt;
			myslave.pongwait = 1;
			}
		}


/* This next part will turn off the lamp automatically after it has been
on for 3 minutes straight, to avoid leaving it on all night long. */

	checktimeouts(&rccar);
	deletecamcounts();
	

// the old lamp code:
	if(officesd.status && time(NULL) > 3*60+officesd.counter)
		{
		lampoff(&officesd,getstatevar(officesd.switchbase));
		}



	if(time(NULL) - insanitytime >= 10)
		{
		insanity = insanity - inssub;
		insanitytime = time(NULL);
		if(insanity < 0)
			insanity=0;
		}
	if(slavedelay != 0 && slavedelay < time(NULL))
		{
		slave_configure_lamps(0);
		slavedelay=0;
		}


	if(sprinksd.status && time(NULL) > 30+sprinksd.counter)
		{
		lampoff(&sprinksd,getstatevar(sprinksd.switchbase));
		}

/* turn off all the X10 lamps after specified time periods */
	for(z=0;z<16;z++)
		{
		for(y=0;y<17;y++)
			{
			if(X10mods[z][y].control && X10mods[z][y].state && time(NULL) > X10mods[z][y].limit*60+X10mods[z][y].counter)
				{
				printf("Killing.\n");
				x10lampoff2(z,y);
/*
				sprintf(s,"/home/pjm/br %c%d off",z+'A',z);
				system(s);	
				X10mods[z][y].state = 0;
*/
				}
			}
		}

/* once per minute, do a pingtest of all the computers and update the 
compstate array */
	
	if(time(NULL) > compchecktime+60)
		{
		int	xxx5;
		
		for(xxx5=0;xxx5 < 10; xxx5++)
			compstate[xxx5] = 1;
		/*
		compstate[0] = pingtest("spaz");
		compstate[1] = pingtest("inferno");
		compstate[2] = pingtest("node5");
		compstate[3] = pingtest("phat");
		compstate[4] = pingtest("node3");
		compstate[5] = pingtest("freedom");
		compstate[6] = pingtest("doorbell");
		compstate[7] = 1; 
		compstate[8] = pingtest("node4");
		compstate[9] = pingtest("oister");
		*/
		compstate[10] = 0;
		compstate[11] = 0;
		compchecktime = time(NULL);
		}	



/* This next part will center the wheels after they have been turned
 for 5 minutes straight, to avoid running down the transmitter battery. */


	if(rccar.wstatus && time(NULL) > 5*60+rccar.wcounter)
		{
		turn(0,&rccar,&state2);
		}
	} while (1);
}
	




int     pingtest(char   *host)
{
int     x,y,z;
char    s[1024];

sprintf(s,"ping -c 1 %s",host);
x = system(s);
if(x)
        {
	return(0);
        }
else
        {
	return(1);
        }
}                            
